/*                     __                                               *\
**     ________ ___   / /  ___      __ ____  Scala.js Test Suite        **
**    / __/ __// _ | / /  / _ | __ / // __/  (c) 2013, LAMP/EPFL        **
**  __\ \/ /__/ __ |/ /__/ __ |/_// /_\ \    http://scala-js.org/       **
** /____/\___/_/ |_/____/_/ | |__/ /____/                               **
**                          |/____/                                     **
\*                                                                      */
package org.scalajs.testsuite.javalib.lang

import org.junit.Test
import org.junit.Assert._

import org.scalajs.testsuite.utils.AssertThrows._
import org.scalajs.testsuite.utils.Platform._

class CharacterTest {

  @Test def isISOControl(): Unit = {
    val isoControlChars = (('\u0000' to '\u001F') ++
        ('\u007F' to '\u009F')).map(_.toInt).toSet
    isoControlChars foreach { c =>
      assertEquals(true, Character.isISOControl(c))
    }

    val randomInts = List.fill(100)(scala.util.Random.nextInt)
    ((-1000 to 1000) ++ randomInts).filterNot(isoControlChars) foreach { c =>
      assertEquals(false, Character.isISOControl(c))
    }
  }

  @Test def digit(): Unit = {
    assertEquals(10, Character.digit('a', 16))
    assertEquals(-1, Character.digit('}', 5))
    assertEquals(-1, Character.digit('1', 50))
    assertEquals(1, Character.digit('1', 36))
    assertEquals(35, Character.digit('Z', 36))
    assertEquals(11, Character.digit('\uFF22', 20))
  }

  @Test def forDigit(): Unit = {
    /* Ported from
     * https://github.com/gwtproject/gwt/blob/master/user/test/com/google/gwt/emultest/java/lang/CharacterTest.java
     */
    for (i <- 0 until 36) {
      assertEquals(i, Character.digit(Character.forDigit(i, 36), 36))
    }
    assertEquals('9', Character.forDigit(9, 10))
  }

  @Test def toChars(): Unit = {
    assertTrue(Character.toChars(0x61) sameElements Array('a'))
    assertTrue(Character.toChars(0x10000) sameElements Array('\uD800', '\uDC00'))
    assertTrue(Character.toChars(0x10001) sameElements Array('\uD800', '\uDC01'))
    assertTrue(Character.toChars(0x10401) sameElements Array('\uD801', '\uDC01'))
    assertTrue(Character.toChars(0x10FFFF) sameElements Array('\uDBFF', '\uDFFF'))

    expectThrows(classOf[IllegalArgumentException], Character.toChars(Integer.MAX_VALUE))
  }

  @Test def isDigit(): Unit = {
    assertFalse(Character.isDigit('a'))
    assertTrue(Character.isDigit('0'))
    assertTrue(Character.isDigit('5'))
    assertTrue(Character.isDigit('9'))
    assertFalse(Character.isDigit('z'))
    assertFalse(Character.isDigit(' '))

    // 100 randomly chosen characters that produce true
    assertTrue(Character.isDigit('\u0661'))
    assertTrue(Character.isDigit('\u0663'))
    assertTrue(Character.isDigit('\u0664'))
    assertTrue(Character.isDigit('\u0665'))
    assertTrue(Character.isDigit('\u0669'))
    assertTrue(Character.isDigit('\u06F1'))
    assertTrue(Character.isDigit('\u06F3'))
    assertTrue(Character.isDigit('\u06F5'))
    if (!executingInJVMOnJDK6) {
      assertTrue(Character.isDigit('\u07C3'))
      assertTrue(Character.isDigit('\u07C7'))
    }
    assertTrue(Character.isDigit('\u0967'))
    assertTrue(Character.isDigit('\u096C'))
    assertTrue(Character.isDigit('\u096F'))
    assertTrue(Character.isDigit('\u09E6'))
    assertTrue(Character.isDigit('\u09E8'))
    assertTrue(Character.isDigit('\u09EA'))
    assertTrue(Character.isDigit('\u09ED'))
    assertTrue(Character.isDigit('\u0A66'))
    assertTrue(Character.isDigit('\u0A69'))
    assertTrue(Character.isDigit('\u0A6D'))
    assertTrue(Character.isDigit('\u0BE9'))
    assertTrue(Character.isDigit('\u0C67'))
    assertTrue(Character.isDigit('\u0C6A'))
    assertTrue(Character.isDigit('\u0CE6'))
    assertTrue(Character.isDigit('\u0CE9'))
    assertTrue(Character.isDigit('\u0CEC'))
    assertTrue(Character.isDigit('\u0CED'))
    assertTrue(Character.isDigit('\u0CEF'))
    assertTrue(Character.isDigit('\u0D6B'))
    assertTrue(Character.isDigit('\u0D6C'))
    assertTrue(Character.isDigit('\u0E51'))
    assertTrue(Character.isDigit('\u0E53'))
    assertTrue(Character.isDigit('\u0E54'))
    assertTrue(Character.isDigit('\u0ED3'))
    assertTrue(Character.isDigit('\u0ED6'))
    assertTrue(Character.isDigit('\u0F20'))
    assertTrue(Character.isDigit('\u0F21'))
    assertTrue(Character.isDigit('\u1040'))
    assertTrue(Character.isDigit('\u1048'))
    if (!executingInJVMOnJDK6) {
      assertTrue(Character.isDigit('\u1091'))
      assertTrue(Character.isDigit('\u1094'))
    }
    assertTrue(Character.isDigit('\u17E0'))
    assertTrue(Character.isDigit('\u1811'))
    assertTrue(Character.isDigit('\u1814'))
    assertTrue(Character.isDigit('\u1815'))
    assertTrue(Character.isDigit('\u1946'))
    assertTrue(Character.isDigit('\u1947'))
    assertTrue(Character.isDigit('\u1948'))
    assertTrue(Character.isDigit('\u194A'))
    assertTrue(Character.isDigit('\u194C'))
    if (!executingInJVMOnJDK6) {
      assertTrue(Character.isDigit('\u19D4'))
      assertTrue(Character.isDigit('\u19D5'))
      assertTrue(Character.isDigit('\u19D6'))
      assertTrue(Character.isDigit('\u19D9'))
      assertTrue(Character.isDigit('\u1A81'))
      assertTrue(Character.isDigit('\u1A84'))
      assertTrue(Character.isDigit('\u1A86'))
      assertTrue(Character.isDigit('\u1A87'))
      assertTrue(Character.isDigit('\u1A91'))
      assertTrue(Character.isDigit('\u1A92'))
      assertTrue(Character.isDigit('\u1A96'))
      assertTrue(Character.isDigit('\u1A98'))
      assertTrue(Character.isDigit('\u1B56'))
      assertTrue(Character.isDigit('\u1B58'))
      assertTrue(Character.isDigit('\u1BB0'))
      assertTrue(Character.isDigit('\u1BB2'))
      assertTrue(Character.isDigit('\u1C43'))
      assertTrue(Character.isDigit('\u1C51'))
      assertTrue(Character.isDigit('\uA622'))
      assertTrue(Character.isDigit('\uA624'))
      assertTrue(Character.isDigit('\uA8D1'))
      assertTrue(Character.isDigit('\uA8D6'))
      assertTrue(Character.isDigit('\uA8D8'))
      assertTrue(Character.isDigit('\uA9D2'))
      assertTrue(Character.isDigit('\uA9D8'))
      assertTrue(Character.isDigit('\uAA50'))
      assertTrue(Character.isDigit('\uAA52'))
      assertTrue(Character.isDigit('\uAA56'))
      assertTrue(Character.isDigit('\uABF5'))
    }
    assertTrue(Character.isDigit('\uFF12'))
    assertTrue(Character.isDigit('\uFF14'))
    assertTrue(Character.isDigit('\u0033'))

    // 100 randomly chosen characters that produce false
    assertFalse(Character.isDigit('\u04A3'))
    assertFalse(Character.isDigit('\u064B'))
    assertFalse(Character.isDigit('\u078A'))
    assertFalse(Character.isDigit('\u0983'))
    assertFalse(Character.isDigit('\u0AD0'))
    assertFalse(Character.isDigit('\u0B3D'))
    assertFalse(Character.isDigit('\u0E18'))
    assertFalse(Character.isDigit('\u0F53'))
    assertFalse(Character.isDigit('\u1433'))
    assertFalse(Character.isDigit('\u1AD1'))
    assertFalse(Character.isDigit('\u1D41'))
    assertFalse(Character.isDigit('\u1FB6'))
    assertFalse(Character.isDigit('\u2351'))
    assertFalse(Character.isDigit('\u2F86'))
    assertFalse(Character.isDigit('\u3198'))
    assertFalse(Character.isDigit('\u3533'))
    assertFalse(Character.isDigit('\u3FE1'))
    assertFalse(Character.isDigit('\u4717'))
    assertFalse(Character.isDigit('\u47BE'))
    assertFalse(Character.isDigit('\u4A75'))
    assertFalse(Character.isDigit('\u4B4E'))
    assertFalse(Character.isDigit('\u4C18'))
    assertFalse(Character.isDigit('\u531E'))
    assertFalse(Character.isDigit('\u5445'))
    assertFalse(Character.isDigit('\u54CE'))
    assertFalse(Character.isDigit('\u556E'))
    assertFalse(Character.isDigit('\u56A6'))
    assertFalse(Character.isDigit('\u587C'))
    assertFalse(Character.isDigit('\u5972'))
    assertFalse(Character.isDigit('\u5A9A'))
    assertFalse(Character.isDigit('\u5D39'))
    assertFalse(Character.isDigit('\u60CD'))
    assertFalse(Character.isDigit('\u60FC'))
    assertFalse(Character.isDigit('\u611A'))
    assertFalse(Character.isDigit('\u61DC'))
    assertFalse(Character.isDigit('\u61EE'))
    assertFalse(Character.isDigit('\u6750'))
    assertFalse(Character.isDigit('\u683B'))
    assertFalse(Character.isDigit('\u6878'))
    assertFalse(Character.isDigit('\u68A2'))
    assertFalse(Character.isDigit('\u6B6B'))
    assertFalse(Character.isDigit('\u6E83'))
    assertFalse(Character.isDigit('\u7099'))
    assertFalse(Character.isDigit('\u7717'))
    assertFalse(Character.isDigit('\u7AAC'))
    assertFalse(Character.isDigit('\u7B96'))
    assertFalse(Character.isDigit('\u80C3'))
    assertFalse(Character.isDigit('\u828F'))
    assertFalse(Character.isDigit('\u829D'))
    assertFalse(Character.isDigit('\u876B'))
    assertFalse(Character.isDigit('\u8913'))
    assertFalse(Character.isDigit('\u8D2A'))
    assertFalse(Character.isDigit('\u8D82'))
    assertFalse(Character.isDigit('\u8E71'))
    assertFalse(Character.isDigit('\u8F63'))
    assertFalse(Character.isDigit('\u94D9'))
    assertFalse(Character.isDigit('\u9859'))
    assertFalse(Character.isDigit('\u99F9'))
    assertFalse(Character.isDigit('\u9A1A'))
    assertFalse(Character.isDigit('\u9B5C'))
    assertFalse(Character.isDigit('\u9E00'))
    assertFalse(Character.isDigit('\u9E3B'))
    assertFalse(Character.isDigit('\u9E5A'))
    assertFalse(Character.isDigit('\u9F49'))
    assertFalse(Character.isDigit('\u9FED'))
    assertFalse(Character.isDigit('\uA28A'))
    assertFalse(Character.isDigit('\uA6A5'))
    assertFalse(Character.isDigit('\uA84F'))
    assertFalse(Character.isDigit('\uA8A1'))
    assertFalse(Character.isDigit('\uA94C'))
    assertFalse(Character.isDigit('\uA964'))
    assertFalse(Character.isDigit('\uABC1'))
    assertFalse(Character.isDigit('\uAC5A'))
    assertFalse(Character.isDigit('\uAE22'))
    assertFalse(Character.isDigit('\uAE82'))
    assertFalse(Character.isDigit('\uAF4E'))
    assertFalse(Character.isDigit('\uB680'))
    assertFalse(Character.isDigit('\uBC1B'))
    assertFalse(Character.isDigit('\uBDF1'))
    assertFalse(Character.isDigit('\uC1F3'))
    assertFalse(Character.isDigit('\uC71B'))
    assertFalse(Character.isDigit('\uC91F'))
    assertFalse(Character.isDigit('\uC97F'))
    assertFalse(Character.isDigit('\uCA08'))
    assertFalse(Character.isDigit('\uCA4C'))
    assertFalse(Character.isDigit('\uCCEC'))
    assertFalse(Character.isDigit('\uCEA2'))
    assertFalse(Character.isDigit('\uD084'))
    assertFalse(Character.isDigit('\uD24E'))
    assertFalse(Character.isDigit('\uD8F5'))
    assertFalse(Character.isDigit('\uE20A'))
    assertFalse(Character.isDigit('\uED86'))
    assertFalse(Character.isDigit('\uEDF0'))
    assertFalse(Character.isDigit('\uF067'))
    assertFalse(Character.isDigit('\uF121'))
    assertFalse(Character.isDigit('\uF448'))
    assertFalse(Character.isDigit('\uF560'))
    assertFalse(Character.isDigit('\uF79C'))
    assertFalse(Character.isDigit('\uF82D'))
    assertFalse(Character.isDigit('\uFBFC'))
  }

  @Test def compareTo(): Unit = {
    def compare(x: Char, y: Char): Int =
      new Character(x).compareTo(new Character(y))

    assertTrue(compare('0', '5') < 0)
    assertTrue(compare('o', 'g') > 0)
    assertTrue(compare('A', 'a') < 0)
    assertEquals(0, compare('b', 'b'))
  }

  @Test def should_be_a_Comparable(): Unit = {
    def compare(x: Any, y: Any): Int =
      x.asInstanceOf[Comparable[Any]].compareTo(y)

    assertTrue(compare('0', '5') < 0)
    assertTrue(compare('o', 'g') > 0)
    assertTrue(compare('A', 'a') < 0)
    assertEquals(0, compare('b', 'b'))
  }

  @Test def isIdentifierIgnorable(): Unit = {
    for (c <- '\u0000' to '\u0008')
      assertTrue(Character.isIdentifierIgnorable(c))

    for (c <- '\u000E' to '\u001B')
      assertTrue(Character.isIdentifierIgnorable(c))

    for (c <- '\u007F' to '\u009F')
      assertTrue(Character.isIdentifierIgnorable(c))

    // Exhaustive list of Cf category. Unicode 7.0.0
    assertTrue(Character.isIdentifierIgnorable('\u00AD'))
    assertTrue(Character.isIdentifierIgnorable('\u0600'))
    assertTrue(Character.isIdentifierIgnorable('\u0601'))
    assertTrue(Character.isIdentifierIgnorable('\u0602'))
    assertTrue(Character.isIdentifierIgnorable('\u0603'))
    if (!executingInJVM)
      assertTrue(Character.isIdentifierIgnorable('\u0604'))
    // assertTrue(Character.isIdentifierIgnorable('\u0605')) returns false on JVM
    // assertTrue(Character.isIdentifierIgnorable('\u061C')) returns false on JVM
    assertTrue(Character.isIdentifierIgnorable('\u06DD'))
    assertTrue(Character.isIdentifierIgnorable('\u070F'))
    // assertTrue(Character.isIdentifierIgnorable('\u180E'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isIdentifierIgnorable('\u200B'))
    assertTrue(Character.isIdentifierIgnorable('\u200C'))
    assertTrue(Character.isIdentifierIgnorable('\u200D'))
    assertTrue(Character.isIdentifierIgnorable('\u200E'))
    assertTrue(Character.isIdentifierIgnorable('\u200F'))
    assertTrue(Character.isIdentifierIgnorable('\u202A'))
    assertTrue(Character.isIdentifierIgnorable('\u202B'))
    assertTrue(Character.isIdentifierIgnorable('\u202C'))
    assertTrue(Character.isIdentifierIgnorable('\u202D'))
    assertTrue(Character.isIdentifierIgnorable('\u202E'))
    assertTrue(Character.isIdentifierIgnorable('\u2060'))
    assertTrue(Character.isIdentifierIgnorable('\u2061'))
    assertTrue(Character.isIdentifierIgnorable('\u2062'))
    assertTrue(Character.isIdentifierIgnorable('\u2063'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isIdentifierIgnorable('\u2064'))
    // assertTrue(Character.isIdentifierIgnorable('\u2066')) returns false on JVM
    // assertTrue(Character.isIdentifierIgnorable('\u2067')) returns false on JVM
    // assertTrue(Character.isIdentifierIgnorable('\u2068')) returns false on JVM
    // assertTrue(Character.isIdentifierIgnorable('\u2069')) returns false on JVM
    assertTrue(Character.isIdentifierIgnorable('\u206A'))
    assertTrue(Character.isIdentifierIgnorable('\u206B'))
    assertTrue(Character.isIdentifierIgnorable('\u206C'))
    assertTrue(Character.isIdentifierIgnorable('\u206D'))
    assertTrue(Character.isIdentifierIgnorable('\u206E'))
    assertTrue(Character.isIdentifierIgnorable('\u206F'))
    assertTrue(Character.isIdentifierIgnorable('\uFEFF'))
    assertTrue(Character.isIdentifierIgnorable('\uFFF9'))
    assertTrue(Character.isIdentifierIgnorable('\uFFFA'))
    assertTrue(Character.isIdentifierIgnorable('\uFFFB'))

    // BUG in JDK? 17B4 should be "Mn", Java says "Cf"
    //assertTrue(Character.isIdentifierIgnorable('\u17b4'))

    // 100 randomly generated negatives
    assertFalse(Character.isIdentifierIgnorable('\u745a'))
    assertFalse(Character.isIdentifierIgnorable('\ub445'))
    assertFalse(Character.isIdentifierIgnorable('\ub23a'))
    assertFalse(Character.isIdentifierIgnorable('\ub029'))
    assertFalse(Character.isIdentifierIgnorable('\ufb5c'))
    assertFalse(Character.isIdentifierIgnorable('\u1b67'))
    assertFalse(Character.isIdentifierIgnorable('\u943b'))
    assertFalse(Character.isIdentifierIgnorable('\ue766'))
    assertFalse(Character.isIdentifierIgnorable('\uad12'))
    assertFalse(Character.isIdentifierIgnorable('\ub80b'))
    assertFalse(Character.isIdentifierIgnorable('\u7341'))
    assertFalse(Character.isIdentifierIgnorable('\ubc73'))
    assertFalse(Character.isIdentifierIgnorable('\uabb9'))
    assertFalse(Character.isIdentifierIgnorable('\ub34b'))
    assertFalse(Character.isIdentifierIgnorable('\u1063'))
    assertFalse(Character.isIdentifierIgnorable('\u272f'))
    assertFalse(Character.isIdentifierIgnorable('\u3801'))
    assertFalse(Character.isIdentifierIgnorable('\u53a6'))
    assertFalse(Character.isIdentifierIgnorable('\u2ec2'))
    assertFalse(Character.isIdentifierIgnorable('\u540c'))
    assertFalse(Character.isIdentifierIgnorable('\uc85f'))
    assertFalse(Character.isIdentifierIgnorable('\ud2c8'))
    assertFalse(Character.isIdentifierIgnorable('\u551b'))
    assertFalse(Character.isIdentifierIgnorable('\uc0a1'))
    assertFalse(Character.isIdentifierIgnorable('\ud25a'))
    assertFalse(Character.isIdentifierIgnorable('\u2b98'))
    assertFalse(Character.isIdentifierIgnorable('\u398b'))
    assertFalse(Character.isIdentifierIgnorable('\ubc77'))
    assertFalse(Character.isIdentifierIgnorable('\u54cc'))
    assertFalse(Character.isIdentifierIgnorable('\uc9a0'))
    assertFalse(Character.isIdentifierIgnorable('\ud10f'))
    assertFalse(Character.isIdentifierIgnorable('\uf7e1'))
    assertFalse(Character.isIdentifierIgnorable('\u0f29'))
    assertFalse(Character.isIdentifierIgnorable('\uafcd'))
    assertFalse(Character.isIdentifierIgnorable('\uf187'))
    assertFalse(Character.isIdentifierIgnorable('\u6287'))
    assertFalse(Character.isIdentifierIgnorable('\uacb6'))
    assertFalse(Character.isIdentifierIgnorable('\uff99'))
    assertFalse(Character.isIdentifierIgnorable('\ub59e'))
    assertFalse(Character.isIdentifierIgnorable('\uf630'))
    assertFalse(Character.isIdentifierIgnorable('\ufaec'))
    assertFalse(Character.isIdentifierIgnorable('\ua7d7'))
    assertFalse(Character.isIdentifierIgnorable('\u3eab'))
    assertFalse(Character.isIdentifierIgnorable('\u54a5'))
    assertFalse(Character.isIdentifierIgnorable('\u393a'))
    assertFalse(Character.isIdentifierIgnorable('\uc621'))
    assertFalse(Character.isIdentifierIgnorable('\u766c'))
    assertFalse(Character.isIdentifierIgnorable('\ud64c'))
    assertFalse(Character.isIdentifierIgnorable('\u8beb'))
    assertFalse(Character.isIdentifierIgnorable('\u44e2'))
    assertFalse(Character.isIdentifierIgnorable('\ub6f6'))
    assertFalse(Character.isIdentifierIgnorable('\u58b6'))
    assertFalse(Character.isIdentifierIgnorable('\u3bad'))
    assertFalse(Character.isIdentifierIgnorable('\u3c28'))
    assertFalse(Character.isIdentifierIgnorable('\ufbfd'))
    assertFalse(Character.isIdentifierIgnorable('\u585f'))
    assertFalse(Character.isIdentifierIgnorable('\u7227'))
    assertFalse(Character.isIdentifierIgnorable('\ucea7'))
    assertFalse(Character.isIdentifierIgnorable('\u2c82'))
    assertFalse(Character.isIdentifierIgnorable('\u686d'))
    assertFalse(Character.isIdentifierIgnorable('\u120d'))
    assertFalse(Character.isIdentifierIgnorable('\uf3db'))
    assertFalse(Character.isIdentifierIgnorable('\u320a'))
    assertFalse(Character.isIdentifierIgnorable('\ud96e'))
    assertFalse(Character.isIdentifierIgnorable('\u85eb'))
    assertFalse(Character.isIdentifierIgnorable('\u9648'))
    assertFalse(Character.isIdentifierIgnorable('\u08a4'))
    assertFalse(Character.isIdentifierIgnorable('\u9db7'))
    assertFalse(Character.isIdentifierIgnorable('\u82c7'))
    assertFalse(Character.isIdentifierIgnorable('\ufe12'))
    assertFalse(Character.isIdentifierIgnorable('\u0eaf'))
    assertFalse(Character.isIdentifierIgnorable('\u96dc'))
    assertFalse(Character.isIdentifierIgnorable('\u3a2a'))
    assertFalse(Character.isIdentifierIgnorable('\uc72e'))
    assertFalse(Character.isIdentifierIgnorable('\u3745'))
    assertFalse(Character.isIdentifierIgnorable('\ubcf9'))
    assertFalse(Character.isIdentifierIgnorable('\u5f66'))
    assertFalse(Character.isIdentifierIgnorable('\u9be1'))
    assertFalse(Character.isIdentifierIgnorable('\ud81d'))
    assertFalse(Character.isIdentifierIgnorable('\u3ca3'))
    assertFalse(Character.isIdentifierIgnorable('\u3e82'))
    assertFalse(Character.isIdentifierIgnorable('\u7ce4'))
    assertFalse(Character.isIdentifierIgnorable('\u33ca'))
    assertFalse(Character.isIdentifierIgnorable('\ue725'))
    assertFalse(Character.isIdentifierIgnorable('\uef49'))
    assertFalse(Character.isIdentifierIgnorable('\ue2cf'))
    assertFalse(Character.isIdentifierIgnorable('\udcf0'))
    assertFalse(Character.isIdentifierIgnorable('\u5f2e'))
    assertFalse(Character.isIdentifierIgnorable('\u2a63'))
    assertFalse(Character.isIdentifierIgnorable('\ud2d2'))
    assertFalse(Character.isIdentifierIgnorable('\u8023'))
    assertFalse(Character.isIdentifierIgnorable('\ua957'))
    assertFalse(Character.isIdentifierIgnorable('\u10ba'))
    assertFalse(Character.isIdentifierIgnorable('\uf85f'))
    assertFalse(Character.isIdentifierIgnorable('\uc40d'))
    assertFalse(Character.isIdentifierIgnorable('\u2509'))
    assertFalse(Character.isIdentifierIgnorable('\u0d8e'))
    assertFalse(Character.isIdentifierIgnorable('\u9db8'))
    assertFalse(Character.isIdentifierIgnorable('\u824d'))
    assertFalse(Character.isIdentifierIgnorable('\u5670'))
    assertFalse(Character.isIdentifierIgnorable('\u6005'))
    assertFalse(Character.isIdentifierIgnorable('\ub8de'))
    assertFalse(Character.isIdentifierIgnorable('\uff5c'))
    assertFalse(Character.isIdentifierIgnorable('\ub36d'))
    assertFalse(Character.isIdentifierIgnorable('\u0cf2'))
    assertFalse(Character.isIdentifierIgnorable('\u82f6'))
    assertFalse(Character.isIdentifierIgnorable('\u9206'))
    assertFalse(Character.isIdentifierIgnorable('\u95e1'))
    assertFalse(Character.isIdentifierIgnorable('\u990f'))
    assertFalse(Character.isIdentifierIgnorable('\u9fc7'))
    assertFalse(Character.isIdentifierIgnorable('\udffb'))
    assertFalse(Character.isIdentifierIgnorable('\u0ecb'))
    assertFalse(Character.isIdentifierIgnorable('\u7563'))
    assertFalse(Character.isIdentifierIgnorable('\uf0ff'))
    assertFalse(Character.isIdentifierIgnorable('\u6b2e'))
    assertFalse(Character.isIdentifierIgnorable('\u894c'))
    assertFalse(Character.isIdentifierIgnorable('\u8f06'))
    assertFalse(Character.isIdentifierIgnorable('\uffa9'))
    assertFalse(Character.isIdentifierIgnorable('\u37b0'))
    assertFalse(Character.isIdentifierIgnorable('\u3e04'))

  }

  @Test def isUnicodeIdentifierStart(): Unit = {
    // 100 randomly generated positives and 100 randomly generated negatives

    assertTrue(Character.isUnicodeIdentifierStart('\ud6d5'))
    assertTrue(Character.isUnicodeIdentifierStart('\u3f9c'))
    assertTrue(Character.isUnicodeIdentifierStart('\u3a40'))
    assertTrue(Character.isUnicodeIdentifierStart('\u53af'))
    assertTrue(Character.isUnicodeIdentifierStart('\u1636'))
    assertTrue(Character.isUnicodeIdentifierStart('\u4884'))
    assertTrue(Character.isUnicodeIdentifierStart('\ucba4'))
    assertTrue(Character.isUnicodeIdentifierStart('\u1ee4'))
    assertTrue(Character.isUnicodeIdentifierStart('\u6dec'))
    assertTrue(Character.isUnicodeIdentifierStart('\u10d4'))
    assertTrue(Character.isUnicodeIdentifierStart('\u631f'))
    assertTrue(Character.isUnicodeIdentifierStart('\u3661'))
    assertTrue(Character.isUnicodeIdentifierStart('\u55f8'))
    assertTrue(Character.isUnicodeIdentifierStart('\ub4ef'))
    assertTrue(Character.isUnicodeIdentifierStart('\ud509'))
    assertTrue(Character.isUnicodeIdentifierStart('\u65b5'))
    assertTrue(Character.isUnicodeIdentifierStart('\u316b'))
    assertTrue(Character.isUnicodeIdentifierStart('\ub270'))
    assertTrue(Character.isUnicodeIdentifierStart('\u7f0f'))
    assertTrue(Character.isUnicodeIdentifierStart('\uff84'))
    assertTrue(Character.isUnicodeIdentifierStart('\u11cc'))
    assertTrue(Character.isUnicodeIdentifierStart('\u0294'))
    assertTrue(Character.isUnicodeIdentifierStart('\u51b1'))
    assertTrue(Character.isUnicodeIdentifierStart('\u9ae2'))
    assertTrue(Character.isUnicodeIdentifierStart('\u304a'))
    assertTrue(Character.isUnicodeIdentifierStart('\ud5c7'))
    assertTrue(Character.isUnicodeIdentifierStart('\u3b4b'))
    assertTrue(Character.isUnicodeIdentifierStart('\u5e42'))
    assertTrue(Character.isUnicodeIdentifierStart('\u51fc'))
    assertTrue(Character.isUnicodeIdentifierStart('\uc148'))
    assertTrue(Character.isUnicodeIdentifierStart('\uc1ae'))
    assertTrue(Character.isUnicodeIdentifierStart('\u7372'))
    assertTrue(Character.isUnicodeIdentifierStart('\uc116'))
    assertTrue(Character.isUnicodeIdentifierStart('\u5d29'))
    assertTrue(Character.isUnicodeIdentifierStart('\u8753'))
    assertTrue(Character.isUnicodeIdentifierStart('\u50f8'))
    assertTrue(Character.isUnicodeIdentifierStart('\u3f9d'))
    assertTrue(Character.isUnicodeIdentifierStart('\u1f44'))
    assertTrue(Character.isUnicodeIdentifierStart('\ucd43'))
    assertTrue(Character.isUnicodeIdentifierStart('\u9126'))
    assertTrue(Character.isUnicodeIdentifierStart('\u8d2e'))
    assertTrue(Character.isUnicodeIdentifierStart('\u4f5c'))
    assertTrue(Character.isUnicodeIdentifierStart('\u66d7'))
    assertTrue(Character.isUnicodeIdentifierStart('\ua30b'))
    assertTrue(Character.isUnicodeIdentifierStart('\u140b'))
    assertTrue(Character.isUnicodeIdentifierStart('\ub264'))
    assertTrue(Character.isUnicodeIdentifierStart('\u7b35'))
    assertTrue(Character.isUnicodeIdentifierStart('\u15e4'))
    assertTrue(Character.isUnicodeIdentifierStart('\ubb37'))
    assertTrue(Character.isUnicodeIdentifierStart('\u34e3'))
    assertTrue(Character.isUnicodeIdentifierStart('\uac3e'))
    assertTrue(Character.isUnicodeIdentifierStart('\ubd0e'))
    assertTrue(Character.isUnicodeIdentifierStart('\ub641'))
    assertTrue(Character.isUnicodeIdentifierStart('\u1580'))
    assertTrue(Character.isUnicodeIdentifierStart('\u30c1'))
    assertTrue(Character.isUnicodeIdentifierStart('\ub0c8'))
    assertTrue(Character.isUnicodeIdentifierStart('\u8681'))
    assertTrue(Character.isUnicodeIdentifierStart('\u7f14'))
    assertTrue(Character.isUnicodeIdentifierStart('\u4142'))
    assertTrue(Character.isUnicodeIdentifierStart('\u56c1'))
    assertTrue(Character.isUnicodeIdentifierStart('\u0444'))
    assertTrue(Character.isUnicodeIdentifierStart('\u9964'))
    assertTrue(Character.isUnicodeIdentifierStart('\ub5c0'))
    assertTrue(Character.isUnicodeIdentifierStart('\u43d8'))
    assertTrue(Character.isUnicodeIdentifierStart('\u479e'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isUnicodeIdentifierStart('\u0853'))
    assertTrue(Character.isUnicodeIdentifierStart('\ube08'))
    assertTrue(Character.isUnicodeIdentifierStart('\u9346'))
    assertTrue(Character.isUnicodeIdentifierStart('\uf9c1'))
    assertTrue(Character.isUnicodeIdentifierStart('\u0e8a'))
    assertTrue(Character.isUnicodeIdentifierStart('\u212c'))
    assertTrue(Character.isUnicodeIdentifierStart('\u810c'))
    assertTrue(Character.isUnicodeIdentifierStart('\u8089'))
    assertTrue(Character.isUnicodeIdentifierStart('\u1331'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isUnicodeIdentifierStart('\ua5f7'))
    assertTrue(Character.isUnicodeIdentifierStart('\u5e5e'))
    assertTrue(Character.isUnicodeIdentifierStart('\u613b'))
    assertTrue(Character.isUnicodeIdentifierStart('\u34a7'))
    assertTrue(Character.isUnicodeIdentifierStart('\ud15b'))
    assertTrue(Character.isUnicodeIdentifierStart('\uc1fc'))
    assertTrue(Character.isUnicodeIdentifierStart('\u92f1'))
    assertTrue(Character.isUnicodeIdentifierStart('\u3ae6'))
    assertTrue(Character.isUnicodeIdentifierStart('\ufceb'))
    assertTrue(Character.isUnicodeIdentifierStart('\u7584'))
    assertTrue(Character.isUnicodeIdentifierStart('\ufe98'))
    assertTrue(Character.isUnicodeIdentifierStart('\ubb23'))
    assertTrue(Character.isUnicodeIdentifierStart('\u7961'))
    assertTrue(Character.isUnicodeIdentifierStart('\u4445'))
    assertTrue(Character.isUnicodeIdentifierStart('\u4d5f'))
    assertTrue(Character.isUnicodeIdentifierStart('\u61cb'))
    assertTrue(Character.isUnicodeIdentifierStart('\u5176'))
    assertTrue(Character.isUnicodeIdentifierStart('\ub987'))
    assertTrue(Character.isUnicodeIdentifierStart('\u906a'))
    assertTrue(Character.isUnicodeIdentifierStart('\u4317'))
    assertTrue(Character.isUnicodeIdentifierStart('\u93ad'))
    assertTrue(Character.isUnicodeIdentifierStart('\u825a'))
    assertTrue(Character.isUnicodeIdentifierStart('\u7ff8'))
    assertTrue(Character.isUnicodeIdentifierStart('\u533a'))
    assertTrue(Character.isUnicodeIdentifierStart('\u5617'))
    assertTrue(Character.isUnicodeIdentifierStart('\ufcc6'))
    assertFalse(Character.isUnicodeIdentifierStart('\ue398'))
    assertFalse(Character.isUnicodeIdentifierStart('\ueab6'))
    assertFalse(Character.isUnicodeIdentifierStart('\ue7bc'))
    assertFalse(Character.isUnicodeIdentifierStart('\uf8ab'))
    assertFalse(Character.isUnicodeIdentifierStart('\ue27f'))
    assertFalse(Character.isUnicodeIdentifierStart('\uebea'))
    assertFalse(Character.isUnicodeIdentifierStart('\ueedc'))
    assertFalse(Character.isUnicodeIdentifierStart('\uf091'))
    assertFalse(Character.isUnicodeIdentifierStart('\u2785'))
    assertFalse(Character.isUnicodeIdentifierStart('\u287b'))
    assertFalse(Character.isUnicodeIdentifierStart('\uf042'))
    assertFalse(Character.isUnicodeIdentifierStart('\u20f9'))
    assertFalse(Character.isUnicodeIdentifierStart('\u23d6'))
    assertFalse(Character.isUnicodeIdentifierStart('\udc5b'))
    assertFalse(Character.isUnicodeIdentifierStart('\ued16'))
    assertFalse(Character.isUnicodeIdentifierStart('\u1b6b'))
    assertFalse(Character.isUnicodeIdentifierStart('\ue7ba'))
    assertFalse(Character.isUnicodeIdentifierStart('\uf7fa'))
    assertFalse(Character.isUnicodeIdentifierStart('\u2125'))
    assertFalse(Character.isUnicodeIdentifierStart('\uea97'))
    assertFalse(Character.isUnicodeIdentifierStart('\ue624'))
    assertFalse(Character.isUnicodeIdentifierStart('\ufbb8'))
    assertFalse(Character.isUnicodeIdentifierStart('\u2730'))
    assertFalse(Character.isUnicodeIdentifierStart('\udb89'))
    assertFalse(Character.isUnicodeIdentifierStart('\ue30d'))
    assertFalse(Character.isUnicodeIdentifierStart('\u2e24'))
    assertFalse(Character.isUnicodeIdentifierStart('\uf03e'))
    assertFalse(Character.isUnicodeIdentifierStart('\uda27'))
    assertFalse(Character.isUnicodeIdentifierStart('\u28fc'))
    assertFalse(Character.isUnicodeIdentifierStart('\u9ffe'))
    assertFalse(Character.isUnicodeIdentifierStart('\ude19'))
    assertFalse(Character.isUnicodeIdentifierStart('\u0b70'))
    assertFalse(Character.isUnicodeIdentifierStart('\uddfc'))
    assertFalse(Character.isUnicodeIdentifierStart('\ued53'))
    assertFalse(Character.isUnicodeIdentifierStart('\ue8cb'))
    assertFalse(Character.isUnicodeIdentifierStart('\udccc'))
    assertFalse(Character.isUnicodeIdentifierStart('\u00a3'))
    assertFalse(Character.isUnicodeIdentifierStart('\u0bed'))
    assertFalse(Character.isUnicodeIdentifierStart('\u0c68'))
    assertFalse(Character.isUnicodeIdentifierStart('\uf47b'))
    assertFalse(Character.isUnicodeIdentifierStart('\u0f96'))
    assertFalse(Character.isUnicodeIdentifierStart('\ue9c3'))
    assertFalse(Character.isUnicodeIdentifierStart('\uf784'))
    assertFalse(Character.isUnicodeIdentifierStart('\uef4b'))
    assertFalse(Character.isUnicodeIdentifierStart('\udee1'))
    assertFalse(Character.isUnicodeIdentifierStart('\u2f61'))
    assertFalse(Character.isUnicodeIdentifierStart('\uf622'))
    assertFalse(Character.isUnicodeIdentifierStart('\u19f9'))
    assertFalse(Character.isUnicodeIdentifierStart('\ud86a'))
    assertFalse(Character.isUnicodeIdentifierStart('\ued83'))
    assertFalse(Character.isUnicodeIdentifierStart('\uf7e4'))
    assertFalse(Character.isUnicodeIdentifierStart('\uecce'))

    // BUG in JDK? A699 should be "Ll", Java says "Cn"
    // assertFalse(Character.isUnicodeIdentifierStart('\ua699'))

    assertFalse(Character.isUnicodeIdentifierStart('\uaa5f'))
    assertFalse(Character.isUnicodeIdentifierStart('\udf24'))
    assertFalse(Character.isUnicodeIdentifierStart('\u2e0e'))
    assertFalse(Character.isUnicodeIdentifierStart('\uf322'))
    assertFalse(Character.isUnicodeIdentifierStart('\ue137'))
    assertFalse(Character.isUnicodeIdentifierStart('\ued19'))
    assertFalse(Character.isUnicodeIdentifierStart('\u21ab'))
    assertFalse(Character.isUnicodeIdentifierStart('\ue972'))
    assertFalse(Character.isUnicodeIdentifierStart('\udbf2'))
    assertFalse(Character.isUnicodeIdentifierStart('\uf54c'))
    assertFalse(Character.isUnicodeIdentifierStart('\u4dd3'))
    assertFalse(Character.isUnicodeIdentifierStart('\u2769'))
    assertFalse(Character.isUnicodeIdentifierStart('\ue363'))

    // BUG in JDK? 1BBB should be "Lo", Java says "Cn"
    // assertFalse(Character.isUnicodeIdentifierStart('\u1bbb'))

    assertFalse(Character.isUnicodeIdentifierStart('\ueae7'))
    assertFalse(Character.isUnicodeIdentifierStart('\u2bf3'))
    assertFalse(Character.isUnicodeIdentifierStart('\ue704'))
    assertFalse(Character.isUnicodeIdentifierStart('\u1c7f'))
    assertFalse(Character.isUnicodeIdentifierStart('\uf52b'))
    assertFalse(Character.isUnicodeIdentifierStart('\ue9e3'))
    assertFalse(Character.isUnicodeIdentifierStart('\u259b'))
    assertFalse(Character.isUnicodeIdentifierStart('\uf250'))
    assertFalse(Character.isUnicodeIdentifierStart('\uf42f'))
    assertFalse(Character.isUnicodeIdentifierStart('\ue244'))
    assertFalse(Character.isUnicodeIdentifierStart('\u20d9'))
    assertFalse(Character.isUnicodeIdentifierStart('\ua881'))
    assertFalse(Character.isUnicodeIdentifierStart('\u0ee6'))
    assertFalse(Character.isUnicodeIdentifierStart('\u2203'))
    assertFalse(Character.isUnicodeIdentifierStart('\u0fc7'))
    assertFalse(Character.isUnicodeIdentifierStart('\u07fc'))
    assertFalse(Character.isUnicodeIdentifierStart('\udb86'))
    assertFalse(Character.isUnicodeIdentifierStart('\u2a70'))
    assertFalse(Character.isUnicodeIdentifierStart('\u2bb7'))
    assertFalse(Character.isUnicodeIdentifierStart('\uecf0'))
    assertFalse(Character.isUnicodeIdentifierStart('\ude48'))
    assertFalse(Character.isUnicodeIdentifierStart('\u0a3b'))
    assertFalse(Character.isUnicodeIdentifierStart('\u20b8'))
    assertFalse(Character.isUnicodeIdentifierStart('\uf898'))
    assertFalse(Character.isUnicodeIdentifierStart('\u23e6'))
    assertFalse(Character.isUnicodeIdentifierStart('\ud8ba'))
    assertFalse(Character.isUnicodeIdentifierStart('\uda1e'))
    assertFalse(Character.isUnicodeIdentifierStart('\udc12'))
    assertFalse(Character.isUnicodeIdentifierStart('\u2a06'))
    assertFalse(Character.isUnicodeIdentifierStart('\u0888'))
    assertFalse(Character.isUnicodeIdentifierStart('\ud9ec'))
    assertFalse(Character.isUnicodeIdentifierStart('\uf81f'))
    assertFalse(Character.isUnicodeIdentifierStart('\uf817'))
  }

  @Test def isUnicodeIdentifierPart(): Unit = {
    // 100 randomly generated positives and 100 randomly generated negatives

    assertTrue(Character.isUnicodeIdentifierPart('\u48d3'))
    assertTrue(Character.isUnicodeIdentifierPart('\u0905'))
    assertTrue(Character.isUnicodeIdentifierPart('\u8f51'))
    assertTrue(Character.isUnicodeIdentifierPart('\u9bcb'))
    assertTrue(Character.isUnicodeIdentifierPart('\ud358'))
    assertTrue(Character.isUnicodeIdentifierPart('\u1538'))
    assertTrue(Character.isUnicodeIdentifierPart('\uffcf'))
    assertTrue(Character.isUnicodeIdentifierPart('\u83ec'))
    assertTrue(Character.isUnicodeIdentifierPart('\u3a89'))
    assertTrue(Character.isUnicodeIdentifierPart('\ub63a'))
    if (!executingInJVMOnJDK6) {
      assertTrue(Character.isUnicodeIdentifierPart('\ufe24'))
      assertTrue(Character.isUnicodeIdentifierPart('\u2d62'))
    }
    assertTrue(Character.isUnicodeIdentifierPart('\u15ca'))
    assertTrue(Character.isUnicodeIdentifierPart('\u4fa4'))
    assertTrue(Character.isUnicodeIdentifierPart('\u47d1'))
    assertTrue(Character.isUnicodeIdentifierPart('\u831c'))
    assertTrue(Character.isUnicodeIdentifierPart('\u84e6'))
    assertTrue(Character.isUnicodeIdentifierPart('\u7783'))
    assertTrue(Character.isUnicodeIdentifierPart('\ua03c'))
    assertTrue(Character.isUnicodeIdentifierPart('\u6ecf'))
    assertTrue(Character.isUnicodeIdentifierPart('\u147f'))
    assertTrue(Character.isUnicodeIdentifierPart('\u67a9'))
    assertTrue(Character.isUnicodeIdentifierPart('\u8b6c'))
    assertTrue(Character.isUnicodeIdentifierPart('\u3410'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isUnicodeIdentifierPart('\u2cc0'))
    assertTrue(Character.isUnicodeIdentifierPart('\ua332'))
    assertTrue(Character.isUnicodeIdentifierPart('\u9733'))
    assertTrue(Character.isUnicodeIdentifierPart('\u5df3'))
    assertTrue(Character.isUnicodeIdentifierPart('\u3fd7'))
    assertTrue(Character.isUnicodeIdentifierPart('\u6611'))
    assertTrue(Character.isUnicodeIdentifierPart('\u55b4'))
    assertTrue(Character.isUnicodeIdentifierPart('\u8bc8'))
    assertTrue(Character.isUnicodeIdentifierPart('\u6f74'))
    assertTrue(Character.isUnicodeIdentifierPart('\u6c97'))
    assertTrue(Character.isUnicodeIdentifierPart('\u6a86'))
    assertTrue(Character.isUnicodeIdentifierPart('\u6000'))
    assertTrue(Character.isUnicodeIdentifierPart('\u614f'))
    assertTrue(Character.isUnicodeIdentifierPart('\u206e'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isUnicodeIdentifierPart('\ua801'))
    assertTrue(Character.isUnicodeIdentifierPart('\u9edf'))
    assertTrue(Character.isUnicodeIdentifierPart('\ub42c'))
    assertTrue(Character.isUnicodeIdentifierPart('\u7fcd'))
    assertTrue(Character.isUnicodeIdentifierPart('\u8a60'))
    assertTrue(Character.isUnicodeIdentifierPart('\u182f'))
    assertTrue(Character.isUnicodeIdentifierPart('\u5d0a'))
    assertTrue(Character.isUnicodeIdentifierPart('\uaf9c'))
    assertTrue(Character.isUnicodeIdentifierPart('\u9d4b'))
    assertTrue(Character.isUnicodeIdentifierPart('\u5088'))
    assertTrue(Character.isUnicodeIdentifierPart('\uc1a6'))
    assertTrue(Character.isUnicodeIdentifierPart('\ubbe4'))
    assertTrue(Character.isUnicodeIdentifierPart('\uad25'))
    assertTrue(Character.isUnicodeIdentifierPart('\u4653'))
    assertTrue(Character.isUnicodeIdentifierPart('\u8add'))
    assertTrue(Character.isUnicodeIdentifierPart('\u3d1c'))
    assertTrue(Character.isUnicodeIdentifierPart('\u80a8'))
    assertTrue(Character.isUnicodeIdentifierPart('\u810e'))
    assertTrue(Character.isUnicodeIdentifierPart('\uc1d2'))
    assertTrue(Character.isUnicodeIdentifierPart('\ub984'))
    assertTrue(Character.isUnicodeIdentifierPart('\u9d13'))
    assertTrue(Character.isUnicodeIdentifierPart('\u37c2'))
    assertTrue(Character.isUnicodeIdentifierPart('\u13cd'))
    assertTrue(Character.isUnicodeIdentifierPart('\u53f9'))
    assertTrue(Character.isUnicodeIdentifierPart('\u98b7'))
    assertTrue(Character.isUnicodeIdentifierPart('\u57f3'))
    assertTrue(Character.isUnicodeIdentifierPart('\ub554'))
    assertTrue(Character.isUnicodeIdentifierPart('\u0176'))
    assertTrue(Character.isUnicodeIdentifierPart('\ua318'))
    assertTrue(Character.isUnicodeIdentifierPart('\u9704'))
    assertTrue(Character.isUnicodeIdentifierPart('\u8d52'))
    assertTrue(Character.isUnicodeIdentifierPart('\u940a'))
    assertTrue(Character.isUnicodeIdentifierPart('\u0fa5'))
    assertTrue(Character.isUnicodeIdentifierPart('\u38d1'))
    assertTrue(Character.isUnicodeIdentifierPart('\u3b33'))
    assertTrue(Character.isUnicodeIdentifierPart('\u93bb'))
    assertTrue(Character.isUnicodeIdentifierPart('\u03bd'))
    assertTrue(Character.isUnicodeIdentifierPart('\u4c88'))
    assertTrue(Character.isUnicodeIdentifierPart('\ud67d'))
    assertTrue(Character.isUnicodeIdentifierPart('\ubcbf'))
    assertTrue(Character.isUnicodeIdentifierPart('\u3867'))
    assertTrue(Character.isUnicodeIdentifierPart('\u4368'))
    assertTrue(Character.isUnicodeIdentifierPart('\u8f2d'))
    assertTrue(Character.isUnicodeIdentifierPart('\u049a'))
    assertTrue(Character.isUnicodeIdentifierPart('\u4c01'))
    assertTrue(Character.isUnicodeIdentifierPart('\u5589'))
    assertTrue(Character.isUnicodeIdentifierPart('\u5e71'))
    assertTrue(Character.isUnicodeIdentifierPart('\ua1fd'))
    assertTrue(Character.isUnicodeIdentifierPart('\u3a4a'))
    assertTrue(Character.isUnicodeIdentifierPart('\uc111'))
    assertTrue(Character.isUnicodeIdentifierPart('\ub465'))
    assertTrue(Character.isUnicodeIdentifierPart('\u95af'))
    assertTrue(Character.isUnicodeIdentifierPart('\ubf2c'))
    assertTrue(Character.isUnicodeIdentifierPart('\u8488'))
    assertTrue(Character.isUnicodeIdentifierPart('\u4317'))
    assertTrue(Character.isUnicodeIdentifierPart('\u6b77'))
    assertTrue(Character.isUnicodeIdentifierPart('\u8995'))
    assertTrue(Character.isUnicodeIdentifierPart('\u7467'))
    assertTrue(Character.isUnicodeIdentifierPart('\u16b7'))
    assertTrue(Character.isUnicodeIdentifierPart('\u3ca0'))
    assertTrue(Character.isUnicodeIdentifierPart('\u5332'))
    assertTrue(Character.isUnicodeIdentifierPart('\u8654'))
    assertFalse(Character.isUnicodeIdentifierPart('\ua8c8'))
    assertFalse(Character.isUnicodeIdentifierPart('\ue3ca'))
    assertFalse(Character.isUnicodeIdentifierPart('\uebee'))
    assertFalse(Character.isUnicodeIdentifierPart('\u270e'))
    assertFalse(Character.isUnicodeIdentifierPart('\uf0ac'))
    assertFalse(Character.isUnicodeIdentifierPart('\ue9ec'))
    assertFalse(Character.isUnicodeIdentifierPart('\u296a'))
    assertFalse(Character.isUnicodeIdentifierPart('\u33fd'))
    assertFalse(Character.isUnicodeIdentifierPart('\ue5f4'))
    assertFalse(Character.isUnicodeIdentifierPart('\ueb01'))
    assertFalse(Character.isUnicodeIdentifierPart('\uf38b'))
    assertFalse(Character.isUnicodeIdentifierPart('\u2e6f'))
    assertFalse(Character.isUnicodeIdentifierPart('\uea69'))
    assertFalse(Character.isUnicodeIdentifierPart('\uf155'))
    assertFalse(Character.isUnicodeIdentifierPart('\u0f0e'))
    assertFalse(Character.isUnicodeIdentifierPart('\ueb80'))
    assertFalse(Character.isUnicodeIdentifierPart('\ud959'))
    assertFalse(Character.isUnicodeIdentifierPart('\ue25e'))
    assertFalse(Character.isUnicodeIdentifierPart('\uf566'))
    assertFalse(Character.isUnicodeIdentifierPart('\ue4a3'))
    assertFalse(Character.isUnicodeIdentifierPart('\uec44'))
    assertFalse(Character.isUnicodeIdentifierPart('\u3297'))
    assertFalse(Character.isUnicodeIdentifierPart('\u3214'))
    assertFalse(Character.isUnicodeIdentifierPart('\u1bfd'))
    assertFalse(Character.isUnicodeIdentifierPart('\u4dd0'))
    assertFalse(Character.isUnicodeIdentifierPart('\uea99'))
    assertFalse(Character.isUnicodeIdentifierPart('\u309b'))
    assertFalse(Character.isUnicodeIdentifierPart('\uf592'))
    assertFalse(Character.isUnicodeIdentifierPart('\uf4dd'))
    assertFalse(Character.isUnicodeIdentifierPart('\udfaf'))
    assertFalse(Character.isUnicodeIdentifierPart('\udd38'))
    assertFalse(Character.isUnicodeIdentifierPart('\uf820'))
    assertFalse(Character.isUnicodeIdentifierPart('\uaacd'))
    assertFalse(Character.isUnicodeIdentifierPart('\uff5b'))
    assertFalse(Character.isUnicodeIdentifierPart('\ude36'))
    assertFalse(Character.isUnicodeIdentifierPart('\ue33b'))
    assertFalse(Character.isUnicodeIdentifierPart('\udbce'))
    assertFalse(Character.isUnicodeIdentifierPart('\ue1f6'))
    assertFalse(Character.isUnicodeIdentifierPart('\uf78a'))
    assertFalse(Character.isUnicodeIdentifierPart('\ueb44'))
    assertFalse(Character.isUnicodeIdentifierPart('\uebd4'))
    assertFalse(Character.isUnicodeIdentifierPart('\u1df7'))
    assertFalse(Character.isUnicodeIdentifierPart('\u2f10'))
    assertFalse(Character.isUnicodeIdentifierPart('\u1cbf'))
    assertFalse(Character.isUnicodeIdentifierPart('\u2362'))
    assertFalse(Character.isUnicodeIdentifierPart('\uebeb'))
    assertFalse(Character.isUnicodeIdentifierPart('\u2ede'))
    assertFalse(Character.isUnicodeIdentifierPart('\u221d'))
    assertFalse(Character.isUnicodeIdentifierPart('\u2021'))
    assertFalse(Character.isUnicodeIdentifierPart('\udf41'))
    assertFalse(Character.isUnicodeIdentifierPart('\u05f5'))
    assertFalse(Character.isUnicodeIdentifierPart('\u24ab'))
    assertFalse(Character.isUnicodeIdentifierPart('\uee15'))
    assertFalse(Character.isUnicodeIdentifierPart('\uf175'))
    assertFalse(Character.isUnicodeIdentifierPart('\uf35c'))
    assertFalse(Character.isUnicodeIdentifierPart('\udc7b'))
    assertFalse(Character.isUnicodeIdentifierPart('\ud883'))
    assertFalse(Character.isUnicodeIdentifierPart('\uf341'))
    assertFalse(Character.isUnicodeIdentifierPart('\ueec6'))
    assertFalse(Character.isUnicodeIdentifierPart('\u2f57'))
    assertFalse(Character.isUnicodeIdentifierPart('\uff64'))
    assertFalse(Character.isUnicodeIdentifierPart('\ue6a4'))
    assertFalse(Character.isUnicodeIdentifierPart('\uec34'))
    assertFalse(Character.isUnicodeIdentifierPart('\u22a5'))
    assertFalse(Character.isUnicodeIdentifierPart('\uf5ac'))
    assertFalse(Character.isUnicodeIdentifierPart('\u3360'))
    assertFalse(Character.isUnicodeIdentifierPart('\u28b0'))
    assertFalse(Character.isUnicodeIdentifierPart('\uf678'))
    assertFalse(Character.isUnicodeIdentifierPart('\ue0e4'))
    assertFalse(Character.isUnicodeIdentifierPart('\u233f'))
    assertFalse(Character.isUnicodeIdentifierPart('\u0afa'))
    assertFalse(Character.isUnicodeIdentifierPart('\u2013'))
    assertFalse(Character.isUnicodeIdentifierPart('\ud7af'))
    assertFalse(Character.isUnicodeIdentifierPart('\ud98e'))
    assertFalse(Character.isUnicodeIdentifierPart('\ud8a5'))

    // BUG in JDK? A79E should be "Lu", Java says "Cn"
    // assertFalse(Character.isUnicodeIdentifierPart('\ua79e'))

    assertFalse(Character.isUnicodeIdentifierPart('\u1806'))
    assertFalse(Character.isUnicodeIdentifierPart('\ue07a'))
    assertFalse(Character.isUnicodeIdentifierPart('\u2748'))
    assertFalse(Character.isUnicodeIdentifierPart('\uabad'))
    assertFalse(Character.isUnicodeIdentifierPart('\uec5c'))
    assertFalse(Character.isUnicodeIdentifierPart('\ue832'))

    // BUG in JDK? 08A9 should be "Lo", Java says "Cn"
    // assertFalse(Character.isUnicodeIdentifierPart('\u08a9'))

    assertFalse(Character.isUnicodeIdentifierPart('\ue4bd'))
    assertFalse(Character.isUnicodeIdentifierPart('\u208a'))
    assertFalse(Character.isUnicodeIdentifierPart('\uf840'))
    assertFalse(Character.isUnicodeIdentifierPart('\uf570'))
    assertFalse(Character.isUnicodeIdentifierPart('\uef1e'))
    assertFalse(Character.isUnicodeIdentifierPart('\u2bd4'))
    assertFalse(Character.isUnicodeIdentifierPart('\ue385'))
    assertFalse(Character.isUnicodeIdentifierPart('\udc18'))
    assertFalse(Character.isUnicodeIdentifierPart('\u0af0'))
    assertFalse(Character.isUnicodeIdentifierPart('\u244a'))
    assertFalse(Character.isUnicodeIdentifierPart('\uf01e'))
    assertFalse(Character.isUnicodeIdentifierPart('\uf114'))
    assertFalse(Character.isUnicodeIdentifierPart('\ue9c4'))

    // BUG in JDK? AAF4 should be "Lm", Java says "Cn"
    // assertFalse(Character.isUnicodeIdentifierPart('\uaaf4'))

    assertFalse(Character.isUnicodeIdentifierPart('\uf7b9'))
    assertFalse(Character.isUnicodeIdentifierPart('\udd2f'))
    assertFalse(Character.isUnicodeIdentifierPart('\u2d2c'))
  }

  @Test def isUpperCase(): Unit = {
    for (c <- 'a' to 'z')
      assertFalse(Character.isUpperCase(c))
    for (c <- 'A' to 'Z')
      assertTrue(Character.isUpperCase(c))
    for (c <- '0' to '9')
      assertFalse(Character.isUpperCase(c))

    // example form issue #1646
    assertFalse(Character.isUpperCase('('))
    assertFalse(Character.isUpperCase(')'))

    // 100 randomly chosen uppeer cases
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isUpperCase('\uA78B'))
    assertTrue(Character.isUpperCase('\u01D1'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isUpperCase('\uA680'))
    assertTrue(Character.isUpperCase('\u01E2'))
    assertTrue(Character.isUpperCase('\u00C1'))
    assertTrue(Character.isUpperCase('\u038F'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isUpperCase('\u04FC'))
    assertTrue(Character.isUpperCase('\u0232'))
    assertTrue(Character.isUpperCase('\u1E86'))
    assertTrue(Character.isUpperCase('\u1E66'))
    assertTrue(Character.isUpperCase('\u1E12'))
    assertTrue(Character.isUpperCase('\u1F09'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isUpperCase('\uA758'))
    assertTrue(Character.isUpperCase('\u01B1'))
    assertTrue(Character.isUpperCase('\u0230'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isUpperCase('\u2CA2'))
    assertTrue(Character.isUpperCase('\u0154'))
    if (!executingInJVMOnJDK6) {
      assertTrue(Character.isUpperCase('\uA77E'))
      assertTrue(Character.isUpperCase('\u2C69'))
    }
    assertTrue(Character.isUpperCase('\u1EDE'))
    if (!executingInJVMOnJDK6) {
      assertTrue(Character.isUpperCase('\u2CBC'))
      assertTrue(Character.isUpperCase('\u2CED'))
      assertTrue(Character.isUpperCase('\u2CAC'))
    }
    assertTrue(Character.isUpperCase('\u0549'))
    assertTrue(Character.isUpperCase('\u00D5'))
    assertTrue(Character.isUpperCase('\u0494'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isUpperCase('\u2CCC'))
    assertTrue(Character.isUpperCase('\u0162'))
    assertTrue(Character.isUpperCase('\uFF22'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isUpperCase('\u24C4'))
    assertTrue(Character.isUpperCase('\u041A'))
    assertTrue(Character.isUpperCase('\u2124'))
    assertTrue(Character.isUpperCase('\u1FBB'))
    assertTrue(Character.isUpperCase('\u1FD9'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isUpperCase('\uA662'))
    assertTrue(Character.isUpperCase('\u0504'))
    if (!executingInJVMOnJDK6) {
      assertTrue(Character.isUpperCase('\u2C14'))
      assertTrue(Character.isUpperCase('\uA760'))
    }
    assertTrue(Character.isUpperCase('\u1E44'))
    assertTrue(Character.isUpperCase('\u10A2'))
    assertTrue(Character.isUpperCase('\u017B'))
    assertTrue(Character.isUpperCase('\u054D'))
    assertTrue(Character.isUpperCase('\u053E'))
    assertTrue(Character.isUpperCase('\u1E22'))
    assertTrue(Character.isUpperCase('\u013F'))
    assertTrue(Character.isUpperCase('\u1E2A'))
    if (!executingInJVMOnJDK6) {
      assertTrue(Character.isUpperCase('\uA760'))
      assertTrue(Character.isUpperCase('\u24C3'))
    }
    assertTrue(Character.isUpperCase('\u049A'))
    assertTrue(Character.isUpperCase('\u10BD'))
    assertTrue(Character.isUpperCase('\u1EEA'))
    assertTrue(Character.isUpperCase('\u0397'))
    assertTrue(Character.isUpperCase('\u1F3D'))
    assertTrue(Character.isUpperCase('\u050A'))
    assertTrue(Character.isUpperCase('\uFF23'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isUpperCase('\u2C01'))
    assertTrue(Character.isUpperCase('\u10C4'))
    assertTrue(Character.isUpperCase('\u1EA2'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isUpperCase('\u2C72'))
    assertTrue(Character.isUpperCase('\u0554'))
    assertTrue(Character.isUpperCase('\u01E8'))
    assertTrue(Character.isUpperCase('\u10A4'))
    assertTrue(Character.isUpperCase('\u1F0C'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isUpperCase('\u0520'))
    assertTrue(Character.isUpperCase('\u00C5'))
    assertTrue(Character.isUpperCase('\u10AC'))
    if (!executingInJVMOnJDK6) {
      assertTrue(Character.isUpperCase('\u2CA4'))
      assertTrue(Character.isUpperCase('\uA73A'))
      assertTrue(Character.isUpperCase('\u1EFC'))
    }
    assertTrue(Character.isUpperCase('\u0049'))
    assertTrue(Character.isUpperCase('\u10A4'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isUpperCase('\uA652'))
    assertTrue(Character.isUpperCase('\u1E0E'))
    assertTrue(Character.isUpperCase('\u04E2'))
    assertTrue(Character.isUpperCase('\u0174'))
    assertTrue(Character.isUpperCase('\u00CE'))
    assertTrue(Character.isUpperCase('\u042F'))
    assertTrue(Character.isUpperCase('\u053F'))
    assertTrue(Character.isUpperCase('\u0222'))
    assertTrue(Character.isUpperCase('\u1F3F'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isUpperCase('\uA77E'))
    assertTrue(Character.isUpperCase('\u01AC'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isUpperCase('\u2C20'))
    assertTrue(Character.isUpperCase('\u1ECC'))
    assertTrue(Character.isUpperCase('\u04DE'))
    assertTrue(Character.isUpperCase('\u004F'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isUpperCase('\u2CE2'))
    assertTrue(Character.isUpperCase('\u0124'))
    assertTrue(Character.isUpperCase('\u1F4A'))
    assertTrue(Character.isUpperCase('\u0112'))
    assertTrue(Character.isUpperCase('\u016E'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isUpperCase('\u24B7'))
    assertTrue(Character.isUpperCase('\u211A'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isUpperCase('\uA72A'))
    assertTrue(Character.isUpperCase('\u0388'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isUpperCase('\uA726'))
    assertTrue(Character.isUpperCase('\u0052'))
    assertTrue(Character.isUpperCase('\u1EB0'))
    assertTrue(Character.isUpperCase('\u0197'))
    assertTrue(Character.isUpperCase('\u014A'))

    // 100 randomly chosen non uppeer cases
    assertFalse(Character.isUpperCase('\u58C5'))
    assertFalse(Character.isUpperCase('\uF319'))
    assertFalse(Character.isUpperCase('\u1FBC'))
    assertFalse(Character.isUpperCase('\u8143'))
    assertFalse(Character.isUpperCase('\u468A'))
    assertFalse(Character.isUpperCase('\uEAA8'))
    assertFalse(Character.isUpperCase('\u3D3A'))
    assertFalse(Character.isUpperCase('\uB168'))
    assertFalse(Character.isUpperCase('\u6747'))
    assertFalse(Character.isUpperCase('\uAA8D'))
    assertFalse(Character.isUpperCase('\u26B7'))
    assertFalse(Character.isUpperCase('\uDFE3'))
    assertFalse(Character.isUpperCase('\u51AD'))
    assertFalse(Character.isUpperCase('\uB4FC'))
    assertFalse(Character.isUpperCase('\uE23A'))
    assertFalse(Character.isUpperCase('\u9465'))
    assertFalse(Character.isUpperCase('\uAE66'))
    assertFalse(Character.isUpperCase('\u2EA3'))
    assertFalse(Character.isUpperCase('\u2D05'))
    assertFalse(Character.isUpperCase('\u2033'))
    assertFalse(Character.isUpperCase('\u4BFA'))
    assertFalse(Character.isUpperCase('\u89B6'))
    assertFalse(Character.isUpperCase('\u9A3F'))
    assertFalse(Character.isUpperCase('\u4FCD'))
    assertFalse(Character.isUpperCase('\u770B'))
    assertFalse(Character.isUpperCase('\u17B4'))
    assertFalse(Character.isUpperCase('\u08A6'))
    assertFalse(Character.isUpperCase('\uA202'))
    assertFalse(Character.isUpperCase('\uE726'))
    assertFalse(Character.isUpperCase('\u8605'))
    assertFalse(Character.isUpperCase('\uE24F'))
    assertFalse(Character.isUpperCase('\u588C'))
    assertFalse(Character.isUpperCase('\u3AF6'))
    assertFalse(Character.isUpperCase('\u313F'))
    assertFalse(Character.isUpperCase('\u8804'))
    assertFalse(Character.isUpperCase('\uE02A'))
    assertFalse(Character.isUpperCase('\u89C5'))
    assertFalse(Character.isUpperCase('\u3B9E'))
    assertFalse(Character.isUpperCase('\uF678'))
    assertFalse(Character.isUpperCase('\uD142'))
    assertFalse(Character.isUpperCase('\u19E5'))
    assertFalse(Character.isUpperCase('\u8C7A'))
    assertFalse(Character.isUpperCase('\uBF0C'))
    assertFalse(Character.isUpperCase('\u5EC6'))
    assertFalse(Character.isUpperCase('\u6EA3'))
    assertFalse(Character.isUpperCase('\uCD19'))
    assertFalse(Character.isUpperCase('\uA6F3'))
    assertFalse(Character.isUpperCase('\uA5FD'))
    assertFalse(Character.isUpperCase('\uEAA7'))
    assertFalse(Character.isUpperCase('\u89DB'))
    assertFalse(Character.isUpperCase('\u2657'))
    assertFalse(Character.isUpperCase('\u667A'))
    assertFalse(Character.isUpperCase('\u131C'))
    assertFalse(Character.isUpperCase('\u5906'))
    assertFalse(Character.isUpperCase('\uAA61'))
    assertFalse(Character.isUpperCase('\uCDD3'))
    assertFalse(Character.isUpperCase('\u1952'))
    assertFalse(Character.isUpperCase('\uD93B'))
    assertFalse(Character.isUpperCase('\uE5B4'))
    assertFalse(Character.isUpperCase('\uF5D1'))
    assertFalse(Character.isUpperCase('\uFC49'))
    assertFalse(Character.isUpperCase('\u51D6'))
    assertFalse(Character.isUpperCase('\u1D29'))
    assertFalse(Character.isUpperCase('\u2A30'))
    assertFalse(Character.isUpperCase('\u881A'))
    assertFalse(Character.isUpperCase('\u2EA5'))
    assertFalse(Character.isUpperCase('\u866C'))
    assertFalse(Character.isUpperCase('\uCAF5'))
    assertFalse(Character.isUpperCase('\uF5F1'))
    assertFalse(Character.isUpperCase('\u97B4'))
    assertFalse(Character.isUpperCase('\uF5F5'))
    assertFalse(Character.isUpperCase('\u46FD'))
    assertFalse(Character.isUpperCase('\u538B'))
    assertFalse(Character.isUpperCase('\u320B'))
    assertFalse(Character.isUpperCase('\u6F1F'))
    assertFalse(Character.isUpperCase('\u85B4'))
    assertFalse(Character.isUpperCase('\u887E'))
    assertFalse(Character.isUpperCase('\uEA84'))
    assertFalse(Character.isUpperCase('\uFFE2'))
    assertFalse(Character.isUpperCase('\u48B2'))
    assertFalse(Character.isUpperCase('\u3D8D'))
    assertFalse(Character.isUpperCase('\u071B'))
    assertFalse(Character.isUpperCase('\u70D9'))
    assertFalse(Character.isUpperCase('\u86FD'))
    assertFalse(Character.isUpperCase('\u9303'))
    assertFalse(Character.isUpperCase('\u6620'))
    assertFalse(Character.isUpperCase('\uB503'))
    assertFalse(Character.isUpperCase('\u9D5A'))
    assertFalse(Character.isUpperCase('\u7909'))
    assertFalse(Character.isUpperCase('\u772C'))
    assertFalse(Character.isUpperCase('\u93D8'))
    assertFalse(Character.isUpperCase('\uE969'))
    assertFalse(Character.isUpperCase('\u327F'))
    assertFalse(Character.isUpperCase('\u8473'))
    assertFalse(Character.isUpperCase('\u478B'))
    assertFalse(Character.isUpperCase('\uB916'))
    assertFalse(Character.isUpperCase('\uBBFA'))
    assertFalse(Character.isUpperCase('\uDC65'))
    assertFalse(Character.isUpperCase('\u65E9'))
    assertFalse(Character.isUpperCase('\u97FF'))
  }

  @Test def isLowerCase(): Unit = {
    for (c <- 'a' to 'z')
      assertTrue(Character.isLowerCase(c))
    for (c <- 'A' to 'Z')
      assertFalse(Character.isLowerCase(c))
    for (c <- '0' to '9')
      assertFalse(Character.isLowerCase(c))

    // example form issue #1646
    assertFalse(Character.isLowerCase('('))
    assertFalse(Character.isLowerCase(')'))

    // 50 randomly chosen characters that produce true
    assertTrue(Character.isLowerCase('\u00BA'))
    if (!executingInJVMOnJDK6) {
      assertTrue(Character.isLowerCase('\u02B0'))
      assertTrue(Character.isLowerCase('\u02B2'))
      assertTrue(Character.isLowerCase('\u02E1'))
      assertTrue(Character.isLowerCase('\u02E3'))
      assertTrue(Character.isLowerCase('\u1D2D'))
      assertTrue(Character.isLowerCase('\u1D34'))
      assertTrue(Character.isLowerCase('\u1D36'))
      assertTrue(Character.isLowerCase('\u1D3B'))
      assertTrue(Character.isLowerCase('\u1D3D'))
      assertTrue(Character.isLowerCase('\u1D42'))
      assertTrue(Character.isLowerCase('\u1D43'))
      assertTrue(Character.isLowerCase('\u1D48'))
      assertTrue(Character.isLowerCase('\u1D4C'))
      assertTrue(Character.isLowerCase('\u1D51'))
      assertTrue(Character.isLowerCase('\u1D54'))
      assertTrue(Character.isLowerCase('\u1D61'))
    }
    assertTrue(Character.isLowerCase('\u1D64'))
    assertTrue(Character.isLowerCase('\u1D65'))
    assertTrue(Character.isLowerCase('\u1D66'))
    assertTrue(Character.isLowerCase('\u1D67'))
    if (!executingInJVMOnJDK6) {
      assertTrue(Character.isLowerCase('\u1D9B'))
      assertTrue(Character.isLowerCase('\u1D9C'))
      assertTrue(Character.isLowerCase('\u1DA2'))
      assertTrue(Character.isLowerCase('\u1DAB'))
      assertTrue(Character.isLowerCase('\u1DB0'))
      assertTrue(Character.isLowerCase('\u1DB2'))
      assertTrue(Character.isLowerCase('\u1DBA'))
      assertTrue(Character.isLowerCase('\u1DBB'))
      assertTrue(Character.isLowerCase('\u2092'))
    }
    if (!executingInJVMOnJDK7OrLower) {
      // Fail on JDK7 in the JVM
      assertTrue(Character.isLowerCase('\u2096'))
      assertTrue(Character.isLowerCase('\u2098'))
      assertTrue(Character.isLowerCase('\u2099'))
    }
    if (!executingInJVMOnJDK6) {
      assertTrue(Character.isLowerCase('\u2173'))
      assertTrue(Character.isLowerCase('\u2176'))
      assertTrue(Character.isLowerCase('\u2177'))
      assertTrue(Character.isLowerCase('\u217D'))
      assertTrue(Character.isLowerCase('\u217F'))
      assertTrue(Character.isLowerCase('\u24D5'))
      assertTrue(Character.isLowerCase('\u24D9'))
      assertTrue(Character.isLowerCase('\u24DE'))
      assertTrue(Character.isLowerCase('\u24E4'))
    }

    // 50 randomly chosen characters that produce false
    assertFalse(Character.isLowerCase('\u0EB1'))
    assertFalse(Character.isLowerCase('\u8081'))
    assertFalse(Character.isLowerCase('\u9CC7'))
    assertFalse(Character.isLowerCase('\uF690'))
    assertFalse(Character.isLowerCase(1000364))
    assertFalse(Character.isLowerCase(1014512))
    assertFalse(Character.isLowerCase(1015728))
    assertFalse(Character.isLowerCase(1059585))
    assertFalse(Character.isLowerCase(1060648))
    assertFalse(Character.isLowerCase(1072459))
    assertFalse(Character.isLowerCase(158456))
    assertFalse(Character.isLowerCase(202025))
    assertFalse(Character.isLowerCase(210769))
    assertFalse(Character.isLowerCase(229801))
    assertFalse(Character.isLowerCase(297040))
    assertFalse(Character.isLowerCase(336465))
    assertFalse(Character.isLowerCase(371554))
    assertFalse(Character.isLowerCase(375989))
    assertFalse(Character.isLowerCase(400272))
    assertFalse(Character.isLowerCase(416137))
    assertFalse(Character.isLowerCase(438321))
    assertFalse(Character.isLowerCase(440405))
    assertFalse(Character.isLowerCase(443020))
    assertFalse(Character.isLowerCase(459145))
    assertFalse(Character.isLowerCase(514186))
    assertFalse(Character.isLowerCase(540856))
    assertFalse(Character.isLowerCase(596077))
    assertFalse(Character.isLowerCase(599610))
    assertFalse(Character.isLowerCase(606387))
    assertFalse(Character.isLowerCase(669554))
    assertFalse(Character.isLowerCase(707507))
    assertFalse(Character.isLowerCase(712083))
    assertFalse(Character.isLowerCase(725257))
    assertFalse(Character.isLowerCase(742132))
    assertFalse(Character.isLowerCase(767248))
    assertFalse(Character.isLowerCase(767605))
    assertFalse(Character.isLowerCase(787060))
    assertFalse(Character.isLowerCase(801428))
    assertFalse(Character.isLowerCase(816223))
    assertFalse(Character.isLowerCase(858580))
    assertFalse(Character.isLowerCase(870913))
    assertFalse(Character.isLowerCase(907591))
    assertFalse(Character.isLowerCase(911179))
    assertFalse(Character.isLowerCase(919865))
    assertFalse(Character.isLowerCase(923549))
    assertFalse(Character.isLowerCase(932196))
    assertFalse(Character.isLowerCase(947222))
    assertFalse(Character.isLowerCase(967074))
    assertFalse(Character.isLowerCase(968024))
    assertFalse(Character.isLowerCase(989574))
  }

  @Test def isLetter(): Unit = {
    for (c <- 'a' to 'z')
      assertTrue(Character.isLetter(c))
    for (c <- 'A' to 'Z')
      assertTrue(Character.isLetter(c))

    // 100 randomly chosen characters that produce true
    assertTrue(Character.isLetter('\u0931'))
    assertTrue(Character.isLetter('\u0E2D'))
    assertTrue(Character.isLetter('\u1E4F'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isLetter('\u2DA5'))
    assertTrue(Character.isLetter('\u36AF'))
    assertTrue(Character.isLetter('\u38BE'))
    assertTrue(Character.isLetter('\u38DD'))
    assertTrue(Character.isLetter('\u39CE'))
    assertTrue(Character.isLetter('\u3D44'))
    assertTrue(Character.isLetter('\u3D63'))
    assertTrue(Character.isLetter('\u4821'))
    assertTrue(Character.isLetter('\u53F3'))
    assertTrue(Character.isLetter('\u5860'))
    assertTrue(Character.isLetter('\u588E'))
    assertTrue(Character.isLetter('\u58FE'))
    assertTrue(Character.isLetter('\u596C'))
    assertTrue(Character.isLetter('\u59DE'))
    assertTrue(Character.isLetter('\u670F'))
    assertTrue(Character.isLetter('\u6E9A'))
    assertTrue(Character.isLetter('\u7072'))
    assertTrue(Character.isLetter('\u7542'))
    assertTrue(Character.isLetter('\u8145'))
    assertTrue(Character.isLetter('\u8494'))
    assertTrue(Character.isLetter('\u8495'))
    assertTrue(Character.isLetter('\u8A95'))
    assertTrue(Character.isLetter('\u8F1C'))
    assertTrue(Character.isLetter('\u8FBB'))
    assertTrue(Character.isLetter('\u9134'))
    assertTrue(Character.isLetter('\u9503'))
    assertTrue(Character.isLetter('\u951E'))
    assertTrue(Character.isLetter('\u9EBA'))
    assertTrue(Character.isLetter('\uA13C'))
    if (!executingInJVMOnJDK6) {
      assertTrue(Character.isLetter('\uA5E6'))
      assertTrue(Character.isLetter('\uA810'))
    }
    assertTrue(Character.isLetter('\uB3B5'))
    assertTrue(Character.isLetter('\uB4BA'))
    assertTrue(Character.isLetter('\uB66A'))
    assertTrue(Character.isLetter('\uCC72'))
    assertTrue(Character.isLetter('\uCFC5'))
    assertTrue(Character.isLetter('\uD0EA'))
    assertTrue(Character.isLetter('\uD3DB'))
    assertTrue(Character.isLetter('\uD421'))
    assertTrue(Character.isLetter('\uD4C7'))
    assertTrue(Character.isLetter('\uD5C9'))
    assertTrue(Character.isLetter('\uD645'))
    assertTrue(Character.isLetter('\uFEEC'))
    if (!executingInJVMOnJDK7OrLower)
      assertTrue(Character.isLetter(126614))
    assertTrue(Character.isLetter(131284))
    assertTrue(Character.isLetter(132125))
    assertTrue(Character.isLetter(133027))
    assertTrue(Character.isLetter(134132))
    assertTrue(Character.isLetter(135009))
    assertTrue(Character.isLetter(136707))
    assertTrue(Character.isLetter(137302))
    assertTrue(Character.isLetter(139777))
    assertTrue(Character.isLetter(139817))
    assertTrue(Character.isLetter(140291))
    assertTrue(Character.isLetter(140662))
    assertTrue(Character.isLetter(141121))
    assertTrue(Character.isLetter(142226))
    assertTrue(Character.isLetter(142357))
    assertTrue(Character.isLetter(143348))
    assertTrue(Character.isLetter(143406))
    assertTrue(Character.isLetter(144320))
    assertTrue(Character.isLetter(150432))
    assertTrue(Character.isLetter(150647))
    assertTrue(Character.isLetter(150718))
    assertTrue(Character.isLetter(154065))
    assertTrue(Character.isLetter(154216))
    assertTrue(Character.isLetter(154406))
    assertTrue(Character.isLetter(154981))
    assertTrue(Character.isLetter(156075))
    assertTrue(Character.isLetter(156323))
    assertTrue(Character.isLetter(156813))
    assertTrue(Character.isLetter(157428))
    assertTrue(Character.isLetter(159375))
    assertTrue(Character.isLetter(159743))
    assertTrue(Character.isLetter(160356))
    assertTrue(Character.isLetter(160611))
    assertTrue(Character.isLetter(160671))
    assertTrue(Character.isLetter(161921))
    assertTrue(Character.isLetter(163295))
    assertTrue(Character.isLetter(163637))
    assertTrue(Character.isLetter(163690))
    assertTrue(Character.isLetter(164083))
    assertTrue(Character.isLetter(166120))
    assertTrue(Character.isLetter(166691))
    assertTrue(Character.isLetter(167425))
    assertTrue(Character.isLetter(168930))
    assertTrue(Character.isLetter(169746))
    assertTrue(Character.isLetter(171107))
    assertTrue(Character.isLetter(171334))
    assertTrue(Character.isLetter(172074))
    assertTrue(Character.isLetter(173301))
    if (!executingInJVMOnJDK6) {
      assertTrue(Character.isLetter(175989))
      assertTrue(Character.isLetter(176347))
      assertTrue(Character.isLetter(176935))
      assertTrue(Character.isLetter(66253))
      assertTrue(Character.isLetter(74015))
    }
    if (!executingInJVMOnJDK7OrLower)
      assertTrue(Character.isLetter(93997))

    // 100 randomly chosen characters that produce false
    assertFalse(Character.isLetter('\uE033'))
    assertFalse(Character.isLetter('\uF45A'))
    assertFalse(Character.isLetter(1015529))
    assertFalse(Character.isLetter(1037692))
    assertFalse(Character.isLetter(1054190))
    assertFalse(Character.isLetter(1055278))
    assertFalse(Character.isLetter(1063915))
    assertFalse(Character.isLetter(1081392))
    assertFalse(Character.isLetter(1082345))
    assertFalse(Character.isLetter(1112137))
    assertFalse(Character.isLetter(112424))
    assertFalse(Character.isLetter(119118))
    assertFalse(Character.isLetter(127185))
    assertFalse(Character.isLetter(181693))
    assertFalse(Character.isLetter(185568))
    assertFalse(Character.isLetter(191510))
    assertFalse(Character.isLetter(205741))
    assertFalse(Character.isLetter(208447))
    assertFalse(Character.isLetter(217327))
    assertFalse(Character.isLetter(223543))
    assertFalse(Character.isLetter(226707))
    assertFalse(Character.isLetter(250211))
    assertFalse(Character.isLetter(252710))
    assertFalse(Character.isLetter(270091))
    assertFalse(Character.isLetter(275467))
    assertFalse(Character.isLetter(282441))
    assertFalse(Character.isLetter(289827))
    assertFalse(Character.isLetter(293540))
    assertFalse(Character.isLetter(306340))
    assertFalse(Character.isLetter(310398))
    assertFalse(Character.isLetter(313871))
    assertFalse(Character.isLetter(325422))
    assertFalse(Character.isLetter(330320))
    assertFalse(Character.isLetter(331194))
    assertFalse(Character.isLetter(353651))
    assertFalse(Character.isLetter(354486))
    assertFalse(Character.isLetter(358669))
    assertFalse(Character.isLetter(359223))
    assertFalse(Character.isLetter(369175))
    assertFalse(Character.isLetter(401793))
    assertFalse(Character.isLetter(427440))
    assertFalse(Character.isLetter(429539))
    assertFalse(Character.isLetter(442272))
    assertFalse(Character.isLetter(447571))
    assertFalse(Character.isLetter(465381))
    assertFalse(Character.isLetter(471567))
    assertFalse(Character.isLetter(480063))
    assertFalse(Character.isLetter(485290))
    assertFalse(Character.isLetter(494999))
    assertFalse(Character.isLetter(506870))
    assertFalse(Character.isLetter(518359))
    assertFalse(Character.isLetter(535534))
    assertFalse(Character.isLetter(539223))
    assertFalse(Character.isLetter(545906))
    assertFalse(Character.isLetter(548645))
    assertFalse(Character.isLetter(555310))
    assertFalse(Character.isLetter(557086))
    assertFalse(Character.isLetter(557887))
    assertFalse(Character.isLetter(564146))
    assertFalse(Character.isLetter(565150))
    assertFalse(Character.isLetter(567248))
    assertFalse(Character.isLetter(567319))
    assertFalse(Character.isLetter(572628))
    assertFalse(Character.isLetter(578097))
    assertFalse(Character.isLetter(604215))
    assertFalse(Character.isLetter(627437))
    assertFalse(Character.isLetter(645283))
    assertFalse(Character.isLetter(645827))
    assertFalse(Character.isLetter(646383))
    assertFalse(Character.isLetter(690854))
    assertFalse(Character.isLetter(695409))
    assertFalse(Character.isLetter(698090))
    assertFalse(Character.isLetter(703125))
    assertFalse(Character.isLetter(713953))
    assertFalse(Character.isLetter(723747))
    assertFalse(Character.isLetter(729300))
    assertFalse(Character.isLetter(732165))
    assertFalse(Character.isLetter(732485))
    assertFalse(Character.isLetter(737696))
    assertFalse(Character.isLetter(742207))
    assertFalse(Character.isLetter(743318))
    assertFalse(Character.isLetter(749325))
    assertFalse(Character.isLetter(761900))
    assertFalse(Character.isLetter(763767))
    assertFalse(Character.isLetter(769676))
    assertFalse(Character.isLetter(775095))
    assertFalse(Character.isLetter(792880))
    assertFalse(Character.isLetter(797399))
    assertFalse(Character.isLetter(80326))
    assertFalse(Character.isLetter(816642))
    assertFalse(Character.isLetter(837600))
    assertFalse(Character.isLetter(839970))
    assertFalse(Character.isLetter(844029))
    assertFalse(Character.isLetter(86861))
    assertFalse(Character.isLetter(873152))
    assertFalse(Character.isLetter(884430))
    assertFalse(Character.isLetter(900180))
    assertFalse(Character.isLetter(901875))
    assertFalse(Character.isLetter(916884))
    assertFalse(Character.isLetter(983136))
  }

  @Test def isLetterOrDigit(): Unit = {
    // 100 randomly chosen characters that produce true
    assertTrue(Character.isLetterOrDigit('\u0209'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isLetterOrDigit('\u0377'))
    assertTrue(Character.isLetterOrDigit('\u0497'))
    assertTrue(Character.isLetterOrDigit('\u053E'))
    assertTrue(Character.isLetterOrDigit('\u0726'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isLetterOrDigit('\u075A'))
    assertTrue(Character.isLetterOrDigit('\u0A91'))
    assertTrue(Character.isLetterOrDigit('\u12CB'))
    assertTrue(Character.isLetterOrDigit('\u182B'))
    assertTrue(Character.isLetterOrDigit('\u1866'))
    assertTrue(Character.isLetterOrDigit('\u1F1C'))
    assertTrue(Character.isLetterOrDigit('\u1F74'))
    assertTrue(Character.isLetterOrDigit('\u1F7C'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isLetterOrDigit('\u2D53'))
    assertTrue(Character.isLetterOrDigit('\u30DF'))
    assertTrue(Character.isLetterOrDigit('\u3772'))
    assertTrue(Character.isLetterOrDigit('\u3A13'))
    assertTrue(Character.isLetterOrDigit('\u3C1D'))
    assertTrue(Character.isLetterOrDigit('\u3C4C'))
    assertTrue(Character.isLetterOrDigit('\u3DEC'))
    assertTrue(Character.isLetterOrDigit('\u3F1E'))
    assertTrue(Character.isLetterOrDigit('\u3F4E'))
    assertTrue(Character.isLetterOrDigit('\u425A'))
    assertTrue(Character.isLetterOrDigit('\u46D7'))
    assertTrue(Character.isLetterOrDigit('\u4822'))
    assertTrue(Character.isLetterOrDigit('\u4855'))
    assertTrue(Character.isLetterOrDigit('\u485D'))
    assertTrue(Character.isLetterOrDigit('\u4970'))
    assertTrue(Character.isLetterOrDigit('\u4C3A'))
    assertTrue(Character.isLetterOrDigit('\u5188'))
    assertTrue(Character.isLetterOrDigit('\u5412'))
    assertTrue(Character.isLetterOrDigit('\u585A'))
    assertTrue(Character.isLetterOrDigit('\u5933'))
    assertTrue(Character.isLetterOrDigit('\u5966'))
    assertTrue(Character.isLetterOrDigit('\u5B15'))
    assertTrue(Character.isLetterOrDigit('\u5C8B'))
    assertTrue(Character.isLetterOrDigit('\u5DA2'))
    assertTrue(Character.isLetterOrDigit('\u6053'))
    assertTrue(Character.isLetterOrDigit('\u63BB'))
    assertTrue(Character.isLetterOrDigit('\u6476'))
    assertTrue(Character.isLetterOrDigit('\u64B9'))
    assertTrue(Character.isLetterOrDigit('\u65B6'))
    assertTrue(Character.isLetterOrDigit('\u67EB'))
    assertTrue(Character.isLetterOrDigit('\u693B'))
    assertTrue(Character.isLetterOrDigit('\u698C'))
    assertTrue(Character.isLetterOrDigit('\u6B86'))
    assertTrue(Character.isLetterOrDigit('\u6DA1'))
    assertTrue(Character.isLetterOrDigit('\u6E5E'))
    assertTrue(Character.isLetterOrDigit('\u6EFA'))
    assertTrue(Character.isLetterOrDigit('\u73BE'))
    assertTrue(Character.isLetterOrDigit('\u7403'))
    assertTrue(Character.isLetterOrDigit('\u7611'))
    assertTrue(Character.isLetterOrDigit('\u76C9'))
    assertTrue(Character.isLetterOrDigit('\u76F6'))
    assertTrue(Character.isLetterOrDigit('\u7943'))
    assertTrue(Character.isLetterOrDigit('\u7A34'))
    assertTrue(Character.isLetterOrDigit('\u7CA0'))
    assertTrue(Character.isLetterOrDigit('\u7ED2'))
    assertTrue(Character.isLetterOrDigit('\u82F6'))
    assertTrue(Character.isLetterOrDigit('\u83F2'))
    assertTrue(Character.isLetterOrDigit('\u8401'))
    assertTrue(Character.isLetterOrDigit('\u8420'))
    assertTrue(Character.isLetterOrDigit('\u848C'))
    assertTrue(Character.isLetterOrDigit('\u8602'))
    assertTrue(Character.isLetterOrDigit('\u8638'))
    assertTrue(Character.isLetterOrDigit('\u88E7'))
    assertTrue(Character.isLetterOrDigit('\u8A0B'))
    assertTrue(Character.isLetterOrDigit('\u9180'))
    assertTrue(Character.isLetterOrDigit('\u920B'))
    assertTrue(Character.isLetterOrDigit('\u94C9'))
    assertTrue(Character.isLetterOrDigit('\u94CE'))
    assertTrue(Character.isLetterOrDigit('\u95A4'))
    assertTrue(Character.isLetterOrDigit('\u979E'))
    assertTrue(Character.isLetterOrDigit('\u9D66'))
    assertTrue(Character.isLetterOrDigit('\u9ED0'))
    assertTrue(Character.isLetterOrDigit('\uA047'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isLetterOrDigit('\uA65A'))
    assertTrue(Character.isLetterOrDigit('\uAEA3'))
    assertTrue(Character.isLetterOrDigit('\uAEC5'))
    assertTrue(Character.isLetterOrDigit('\uB583'))
    assertTrue(Character.isLetterOrDigit('\uBADA'))
    assertTrue(Character.isLetterOrDigit('\uC56F'))
    assertTrue(Character.isLetterOrDigit('\uC857'))
    assertTrue(Character.isLetterOrDigit('\uC97C'))
    assertTrue(Character.isLetterOrDigit('\uC9BF'))
    assertTrue(Character.isLetterOrDigit('\uCC2B'))
    assertTrue(Character.isLetterOrDigit('\uCC60'))
    assertTrue(Character.isLetterOrDigit('\uCF31'))
    assertTrue(Character.isLetterOrDigit('\uCF9E'))
    assertTrue(Character.isLetterOrDigit('\uD2F6'))
    assertTrue(Character.isLetterOrDigit('\uD353'))
    assertTrue(Character.isLetterOrDigit('\uD415'))
    assertTrue(Character.isLetterOrDigit('\uD4B2'))
    assertTrue(Character.isLetterOrDigit('\uD4D3'))
    assertTrue(Character.isLetterOrDigit('\uD64E'))
    assertTrue(Character.isLetterOrDigit('\uD76B'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isLetterOrDigit('\uFA9F'))
    assertTrue(Character.isLetterOrDigit('\uFB7C'))
    assertTrue(Character.isLetterOrDigit('\uFCDD'))
    assertTrue(Character.isLetterOrDigit('\uFEBD'))

    // 100 randomly chosen characters that produce false
    assertFalse(Character.isLetterOrDigit('\u02D8'))
    assertFalse(Character.isLetterOrDigit('\u0312'))
    assertFalse(Character.isLetterOrDigit('\u0361'))
    assertFalse(Character.isLetterOrDigit('\u05EB'))
    assertFalse(Character.isLetterOrDigit('\u0618'))
    assertFalse(Character.isLetterOrDigit('\u070B'))
    assertFalse(Character.isLetterOrDigit('\u088C'))
    assertFalse(Character.isLetterOrDigit('\u08D7'))
    assertFalse(Character.isLetterOrDigit('\u0947'))
    assertFalse(Character.isLetterOrDigit('\u0B56'))
    assertFalse(Character.isLetterOrDigit('\u0B77'))
    assertFalse(Character.isLetterOrDigit('\u0DDA'))
    assertFalse(Character.isLetterOrDigit('\u0DEB'))
    assertFalse(Character.isLetterOrDigit('\u0E7B'))
    assertFalse(Character.isLetterOrDigit('\u0EBA'))
    assertFalse(Character.isLetterOrDigit('\u0F0B'))
    assertFalse(Character.isLetterOrDigit('\u109B'))
    assertFalse(Character.isLetterOrDigit('\u13F7'))
    assertFalse(Character.isLetterOrDigit('\u181B'))
    assertFalse(Character.isLetterOrDigit('\u193F'))
    assertFalse(Character.isLetterOrDigit('\u1FBF'))
    assertFalse(Character.isLetterOrDigit('\u2103'))
    assertFalse(Character.isLetterOrDigit('\u219A'))
    assertFalse(Character.isLetterOrDigit('\u21C4'))
    assertFalse(Character.isLetterOrDigit('\u2220'))
    assertFalse(Character.isLetterOrDigit('\u2250'))
    assertFalse(Character.isLetterOrDigit('\u228D'))
    assertFalse(Character.isLetterOrDigit('\u231B'))
    assertFalse(Character.isLetterOrDigit('\u23BC'))
    assertFalse(Character.isLetterOrDigit('\u2413'))
    assertFalse(Character.isLetterOrDigit('\u24BF'))
    assertFalse(Character.isLetterOrDigit('\u258F'))
    assertFalse(Character.isLetterOrDigit('\u25C2'))
    assertFalse(Character.isLetterOrDigit('\u27F2'))
    assertFalse(Character.isLetterOrDigit('\u2887'))
    assertFalse(Character.isLetterOrDigit('\u2998'))
    assertFalse(Character.isLetterOrDigit('\u29A5'))
    assertFalse(Character.isLetterOrDigit('\u29D2'))
    assertFalse(Character.isLetterOrDigit('\u2A53'))
    assertFalse(Character.isLetterOrDigit('\u2A70'))
    assertFalse(Character.isLetterOrDigit('\u2B0E'))
    assertFalse(Character.isLetterOrDigit('\u2B5E'))
    assertFalse(Character.isLetterOrDigit('\u2BAF'))
    assertFalse(Character.isLetterOrDigit('\u2EAE'))
    assertFalse(Character.isLetterOrDigit('\u2EB1'))
    assertFalse(Character.isLetterOrDigit('\u32E8'))
    assertFalse(Character.isLetterOrDigit('\uA7B7'))
    assertFalse(Character.isLetterOrDigit('\uA7C3'))
    assertFalse(Character.isLetterOrDigit('\uA837'))
    assertFalse(Character.isLetterOrDigit('\uA8B4'))
    assertFalse(Character.isLetterOrDigit('\uA8B7'))
    assertFalse(Character.isLetterOrDigit('\uAA30'))
    assertFalse(Character.isLetterOrDigit('\uAAEB'))
    assertFalse(Character.isLetterOrDigit('\uD859'))
    assertFalse(Character.isLetterOrDigit('\uDADF'))
    assertFalse(Character.isLetterOrDigit('\uDC4C'))
    assertFalse(Character.isLetterOrDigit('\uDC80'))
    assertFalse(Character.isLetterOrDigit('\uDD5E'))
    assertFalse(Character.isLetterOrDigit('\uDD9C'))
    assertFalse(Character.isLetterOrDigit('\uDDC5'))
    assertFalse(Character.isLetterOrDigit('\uDE97'))
    assertFalse(Character.isLetterOrDigit('\uDF78'))
    assertFalse(Character.isLetterOrDigit('\uDF90'))
    assertFalse(Character.isLetterOrDigit('\uE044'))
    assertFalse(Character.isLetterOrDigit('\uE0D9'))
    assertFalse(Character.isLetterOrDigit('\uE158'))
    assertFalse(Character.isLetterOrDigit('\uE19E'))
    assertFalse(Character.isLetterOrDigit('\uE2AF'))
    assertFalse(Character.isLetterOrDigit('\uE317'))
    assertFalse(Character.isLetterOrDigit('\uE351'))
    assertFalse(Character.isLetterOrDigit('\uE3C6'))
    assertFalse(Character.isLetterOrDigit('\uE47F'))
    assertFalse(Character.isLetterOrDigit('\uE54C'))
    assertFalse(Character.isLetterOrDigit('\uE67A'))
    assertFalse(Character.isLetterOrDigit('\uE69C'))
    assertFalse(Character.isLetterOrDigit('\uE6D1'))
    assertFalse(Character.isLetterOrDigit('\uE6E2'))
    assertFalse(Character.isLetterOrDigit('\uE744'))
    assertFalse(Character.isLetterOrDigit('\uE780'))
    assertFalse(Character.isLetterOrDigit('\uE7A1'))
    assertFalse(Character.isLetterOrDigit('\uE817'))
    assertFalse(Character.isLetterOrDigit('\uE81A'))
    assertFalse(Character.isLetterOrDigit('\uE8DC'))
    assertFalse(Character.isLetterOrDigit('\uE916'))
    assertFalse(Character.isLetterOrDigit('\uEDD3'))
    assertFalse(Character.isLetterOrDigit('\uEF7C'))
    assertFalse(Character.isLetterOrDigit('\uF047'))
    assertFalse(Character.isLetterOrDigit('\uF0D0'))
    assertFalse(Character.isLetterOrDigit('\uF160'))
    assertFalse(Character.isLetterOrDigit('\uF27C'))
    assertFalse(Character.isLetterOrDigit('\uF319'))
    assertFalse(Character.isLetterOrDigit('\uF46B'))
    assertFalse(Character.isLetterOrDigit('\uF594'))
    assertFalse(Character.isLetterOrDigit('\uF6D1'))
    assertFalse(Character.isLetterOrDigit('\uF7F2'))
    assertFalse(Character.isLetterOrDigit('\uF889'))
    assertFalse(Character.isLetterOrDigit('\uF8F6'))
    assertFalse(Character.isLetterOrDigit('\uF8FF'))
    assertFalse(Character.isLetterOrDigit('\uFAE2'))
    assertFalse(Character.isLetterOrDigit('\uFE4B'))
  }

  @Test def isSpaceChar(): Unit = {
    assertTrue(Character.isSpaceChar('\u0020'))
    assertTrue(Character.isSpaceChar('\u00A0'))
    assertTrue(Character.isSpaceChar('\u1680'))
    assertTrue(Character.isSpaceChar('\u180E'))
    assertTrue(Character.isSpaceChar('\u2000'))
    assertTrue(Character.isSpaceChar('\u2001'))
    assertTrue(Character.isSpaceChar('\u2002'))
    assertTrue(Character.isSpaceChar('\u2004'))
    assertTrue(Character.isSpaceChar('\u2005'))
    assertTrue(Character.isSpaceChar('\u2006'))
    assertTrue(Character.isSpaceChar('\u2007'))
    assertTrue(Character.isSpaceChar('\u2008'))
    assertTrue(Character.isSpaceChar('\u2009'))
    assertTrue(Character.isSpaceChar('\u200A'))
    assertTrue(Character.isSpaceChar('\u2028'))
    assertTrue(Character.isSpaceChar('\u2029'))
    assertTrue(Character.isSpaceChar('\u202F'))
    assertTrue(Character.isSpaceChar('\u205F'))
    assertTrue(Character.isSpaceChar('\u3000'))

    // 100 randomly chosen characters that produce false
    assertFalse(Character.isSpaceChar('\u0057'))
    assertFalse(Character.isSpaceChar('\u0290'))
    assertFalse(Character.isSpaceChar('\u0374'))
    assertFalse(Character.isSpaceChar('\u04EA'))
    assertFalse(Character.isSpaceChar('\u0845'))
    assertFalse(Character.isSpaceChar('\u0882'))
    assertFalse(Character.isSpaceChar('\u0956'))
    assertFalse(Character.isSpaceChar('\u0979'))
    assertFalse(Character.isSpaceChar('\u0A78'))
    assertFalse(Character.isSpaceChar('\u0D6B'))
    assertFalse(Character.isSpaceChar('\u122F'))
    assertFalse(Character.isSpaceChar('\u137B'))
    assertFalse(Character.isSpaceChar('\u1690'))
    assertFalse(Character.isSpaceChar('\u1875'))
    assertFalse(Character.isSpaceChar('\u1B96'))
    assertFalse(Character.isSpaceChar('\u2253'))
    assertFalse(Character.isSpaceChar('\u2587'))
    assertFalse(Character.isSpaceChar('\u2770'))
    assertFalse(Character.isSpaceChar('\u2975'))
    assertFalse(Character.isSpaceChar('\u304F'))
    assertFalse(Character.isSpaceChar('\u3567'))
    assertFalse(Character.isSpaceChar('\u356A'))
    assertFalse(Character.isSpaceChar('\u35D0'))
    assertFalse(Character.isSpaceChar('\u3625'))
    assertFalse(Character.isSpaceChar('\u389E'))
    assertFalse(Character.isSpaceChar('\u395F'))
    assertFalse(Character.isSpaceChar('\u3A62'))
    assertFalse(Character.isSpaceChar('\u3BBF'))
    assertFalse(Character.isSpaceChar('\u3BF8'))
    assertFalse(Character.isSpaceChar('\u445D'))
    assertFalse(Character.isSpaceChar('\u45C1'))
    assertFalse(Character.isSpaceChar('\u4B46'))
    assertFalse(Character.isSpaceChar('\u4FBF'))
    assertFalse(Character.isSpaceChar('\u54B6'))
    assertFalse(Character.isSpaceChar('\u5A2C'))
    assertFalse(Character.isSpaceChar('\u5A90'))
    assertFalse(Character.isSpaceChar('\u5D28'))
    assertFalse(Character.isSpaceChar('\u5D62'))
    assertFalse(Character.isSpaceChar('\u5ED9'))
    assertFalse(Character.isSpaceChar('\u5FA1'))
    assertFalse(Character.isSpaceChar('\u6242'))
    assertFalse(Character.isSpaceChar('\u6437'))
    assertFalse(Character.isSpaceChar('\u673D'))
    assertFalse(Character.isSpaceChar('\u6AC1'))
    assertFalse(Character.isSpaceChar('\u6CA1'))
    assertFalse(Character.isSpaceChar('\u6D31'))
    assertFalse(Character.isSpaceChar('\u6F7C'))
    assertFalse(Character.isSpaceChar('\u70B3'))
    assertFalse(Character.isSpaceChar('\u74F9'))
    assertFalse(Character.isSpaceChar('\u759E'))
    assertFalse(Character.isSpaceChar('\u76A0'))
    assertFalse(Character.isSpaceChar('\u7792'))
    assertFalse(Character.isSpaceChar('\u814C'))
    assertFalse(Character.isSpaceChar('\u8209'))
    assertFalse(Character.isSpaceChar('\u8451'))
    assertFalse(Character.isSpaceChar('\u86DE'))
    assertFalse(Character.isSpaceChar('\u8878'))
    assertFalse(Character.isSpaceChar('\u8BB5'))
    assertFalse(Character.isSpaceChar('\u8D35'))
    assertFalse(Character.isSpaceChar('\u903B'))
    assertFalse(Character.isSpaceChar('\u907C'))
    assertFalse(Character.isSpaceChar('\u9151'))
    assertFalse(Character.isSpaceChar('\u9188'))
    assertFalse(Character.isSpaceChar('\u9775'))
    assertFalse(Character.isSpaceChar('\u9896'))
    assertFalse(Character.isSpaceChar('\u9C86'))
    assertFalse(Character.isSpaceChar('\u9CB2'))
    assertFalse(Character.isSpaceChar('\uA026'))
    assertFalse(Character.isSpaceChar('\uA34B'))
    assertFalse(Character.isSpaceChar('\uA5B2'))
    assertFalse(Character.isSpaceChar('\uAD41'))
    assertFalse(Character.isSpaceChar('\uAD56'))
    assertFalse(Character.isSpaceChar('\uAE7B'))
    assertFalse(Character.isSpaceChar('\uAFB1'))
    assertFalse(Character.isSpaceChar('\uB1DE'))
    assertFalse(Character.isSpaceChar('\uBC4C'))
    assertFalse(Character.isSpaceChar('\uBE62'))
    assertFalse(Character.isSpaceChar('\uC041'))
    assertFalse(Character.isSpaceChar('\uC24F'))
    assertFalse(Character.isSpaceChar('\uC867'))
    assertFalse(Character.isSpaceChar('\uC8C1'))
    assertFalse(Character.isSpaceChar('\uCA16'))
    assertFalse(Character.isSpaceChar('\uCDDF'))
    assertFalse(Character.isSpaceChar('\uCF2C'))
    assertFalse(Character.isSpaceChar('\uD08D'))
    assertFalse(Character.isSpaceChar('\uD200'))
    assertFalse(Character.isSpaceChar('\uD599'))
    assertFalse(Character.isSpaceChar('\uD7F0'))
    assertFalse(Character.isSpaceChar('\uDBA1'))
    assertFalse(Character.isSpaceChar('\uDBF8'))
    assertFalse(Character.isSpaceChar('\uE066'))
    assertFalse(Character.isSpaceChar('\uE260'))
    assertFalse(Character.isSpaceChar('\uE776'))
    assertFalse(Character.isSpaceChar('\uEA9E'))
    assertFalse(Character.isSpaceChar('\uED5A'))
    assertFalse(Character.isSpaceChar('\uF016'))
    assertFalse(Character.isSpaceChar('\uF2E0'))
    assertFalse(Character.isSpaceChar('\uFA9E'))
    assertFalse(Character.isSpaceChar('\uFB90'))
    assertFalse(Character.isSpaceChar('\uFD88'))
  }

  @Test def isWhitespace(): Unit = {
    // All whitespaces
    assertTrue(Character.isWhitespace('\u0009'))
    assertTrue(Character.isWhitespace('\u000a'))
    assertTrue(Character.isWhitespace('\u000b'))
    assertTrue(Character.isWhitespace('\u000c'))
    assertTrue(Character.isWhitespace('\u000d'))
    assertTrue(Character.isWhitespace('\u001c'))
    assertTrue(Character.isWhitespace('\u001d'))
    assertTrue(Character.isWhitespace('\u001e'))
    assertTrue(Character.isWhitespace('\u001f'))
    assertTrue(Character.isWhitespace('\u0020'))
    assertTrue(Character.isWhitespace('\u1680'))
    assertTrue(Character.isWhitespace('\u180e'))
    assertTrue(Character.isWhitespace('\u2000'))
    assertTrue(Character.isWhitespace('\u2001'))
    assertTrue(Character.isWhitespace('\u2002'))
    assertTrue(Character.isWhitespace('\u2003'))
    assertTrue(Character.isWhitespace('\u2004'))
    assertTrue(Character.isWhitespace('\u2005'))
    assertTrue(Character.isWhitespace('\u2006'))
    assertTrue(Character.isWhitespace('\u2008'))
    assertTrue(Character.isWhitespace('\u2009'))
    assertTrue(Character.isWhitespace('\u200a'))
    assertTrue(Character.isWhitespace('\u2028'))
    assertTrue(Character.isWhitespace('\u2029'))
    assertTrue(Character.isWhitespace('\u205f'))
    assertTrue(Character.isWhitespace('\u3000'))

    if (!executingInJVMOnJDK6)
      assertFalse(Character.isWhitespace('\u200b'))

    // 100 randomly chosen characters that produce false
    assertFalse(Character.isWhitespace(2515))
    assertFalse(Character.isWhitespace(231))
    assertFalse(Character.isWhitespace(63339))
    assertFalse(Character.isWhitespace(60302))
    assertFalse(Character.isWhitespace(56219))
    assertFalse(Character.isWhitespace(9884))
    assertFalse(Character.isWhitespace(23278))
    assertFalse(Character.isWhitespace(16711))
    assertFalse(Character.isWhitespace(29993))
    assertFalse(Character.isWhitespace(29089))
    assertFalse(Character.isWhitespace(42825))
    assertFalse(Character.isWhitespace(57735))
    assertFalse(Character.isWhitespace(21132))
    assertFalse(Character.isWhitespace(40727))
    assertFalse(Character.isWhitespace(21416))
    assertFalse(Character.isWhitespace(8420))
    assertFalse(Character.isWhitespace(46449))
    assertFalse(Character.isWhitespace(56988))
    assertFalse(Character.isWhitespace(60900))
    assertFalse(Character.isWhitespace(6835))
    assertFalse(Character.isWhitespace(17138))
    assertFalse(Character.isWhitespace(50841))
    assertFalse(Character.isWhitespace(1933))
    assertFalse(Character.isWhitespace(32264))
    assertFalse(Character.isWhitespace(56802))
    assertFalse(Character.isWhitespace(10621))
    assertFalse(Character.isWhitespace(63150))
    assertFalse(Character.isWhitespace(60479))
    assertFalse(Character.isWhitespace(2806))
    assertFalse(Character.isWhitespace(56525))
    assertFalse(Character.isWhitespace(51781))
    assertFalse(Character.isWhitespace(7818))
    assertFalse(Character.isWhitespace(12656))
    assertFalse(Character.isWhitespace(47124))
    assertFalse(Character.isWhitespace(52352))
    assertFalse(Character.isWhitespace(54740))
    assertFalse(Character.isWhitespace(27760))
    assertFalse(Character.isWhitespace(57895))
    assertFalse(Character.isWhitespace(11017))
    assertFalse(Character.isWhitespace(28645))
    assertFalse(Character.isWhitespace(46716))
    assertFalse(Character.isWhitespace(6558))
    assertFalse(Character.isWhitespace(8801))
    assertFalse(Character.isWhitespace(16394))
    assertFalse(Character.isWhitespace(44981))
    assertFalse(Character.isWhitespace(29442))
    assertFalse(Character.isWhitespace(1051))
    assertFalse(Character.isWhitespace(21028))
    assertFalse(Character.isWhitespace(21429))
    assertFalse(Character.isWhitespace(24582))
    assertFalse(Character.isWhitespace(3156))
    assertFalse(Character.isWhitespace(391299))
    assertFalse(Character.isWhitespace(555557))
    assertFalse(Character.isWhitespace(104630))
    assertFalse(Character.isWhitespace(311698))
    assertFalse(Character.isWhitespace(775850))
    assertFalse(Character.isWhitespace(472119))
    assertFalse(Character.isWhitespace(927519))
    assertFalse(Character.isWhitespace(345120))
    assertFalse(Character.isWhitespace(372337))
    assertFalse(Character.isWhitespace(415654))
    assertFalse(Character.isWhitespace(1003356))
    assertFalse(Character.isWhitespace(663703))
    assertFalse(Character.isWhitespace(675812))
    assertFalse(Character.isWhitespace(982076))
    assertFalse(Character.isWhitespace(727003))
    assertFalse(Character.isWhitespace(328478))
    assertFalse(Character.isWhitespace(386525))
    assertFalse(Character.isWhitespace(427552))
    assertFalse(Character.isWhitespace(1018963))
    assertFalse(Character.isWhitespace(594357))
    assertFalse(Character.isWhitespace(514150))
    assertFalse(Character.isWhitespace(112164))
    assertFalse(Character.isWhitespace(241410))
    assertFalse(Character.isWhitespace(961291))
    assertFalse(Character.isWhitespace(1098178))
    assertFalse(Character.isWhitespace(911919))
    assertFalse(Character.isWhitespace(867492))
    assertFalse(Character.isWhitespace(233623))
    assertFalse(Character.isWhitespace(641480))
    assertFalse(Character.isWhitespace(536446))
    assertFalse(Character.isWhitespace(557516))
    assertFalse(Character.isWhitespace(511075))
    assertFalse(Character.isWhitespace(195888))
    assertFalse(Character.isWhitespace(808876))
    assertFalse(Character.isWhitespace(841221))
    assertFalse(Character.isWhitespace(90617))
    assertFalse(Character.isWhitespace(872412))
    assertFalse(Character.isWhitespace(307770))
    assertFalse(Character.isWhitespace(307998))
    assertFalse(Character.isWhitespace(373461))
    assertFalse(Character.isWhitespace(149861))
    assertFalse(Character.isWhitespace(747431))
    assertFalse(Character.isWhitespace(992898))
    assertFalse(Character.isWhitespace(1083539))
    assertFalse(Character.isWhitespace(112853))
    assertFalse(Character.isWhitespace(897133))
    assertFalse(Character.isWhitespace(129841))
    assertFalse(Character.isWhitespace(801648))
    assertFalse(Character.isWhitespace(718253))
    assertFalse(Character.isWhitespace(934430))
    assertFalse(Character.isWhitespace(942329))
  }

  @Test def isTitleCase(): Unit = {
    // all tilteCases
    assertTrue(Character.isTitleCase('\u01C5'))
    assertTrue(Character.isTitleCase('\u01C8'))
    assertTrue(Character.isTitleCase('\u01CB'))
    assertTrue(Character.isTitleCase('\u01F2'))
    assertTrue(Character.isTitleCase('\u1F88'))
    assertTrue(Character.isTitleCase('\u1F89'))
    assertTrue(Character.isTitleCase('\u1F8A'))
    assertTrue(Character.isTitleCase('\u1F8B'))
    assertTrue(Character.isTitleCase('\u1F8C'))
    assertTrue(Character.isTitleCase('\u1F8D'))
    assertTrue(Character.isTitleCase('\u1F8E'))
    assertTrue(Character.isTitleCase('\u1F8F'))
    assertTrue(Character.isTitleCase('\u1F98'))
    assertTrue(Character.isTitleCase('\u1F99'))
    assertTrue(Character.isTitleCase('\u1F9A'))
    assertTrue(Character.isTitleCase('\u1F9B'))
    assertTrue(Character.isTitleCase('\u1F9C'))
    assertTrue(Character.isTitleCase('\u1F9D'))
    assertTrue(Character.isTitleCase('\u1F9E'))
    assertTrue(Character.isTitleCase('\u1F9F'))
    assertTrue(Character.isTitleCase('\u1FA8'))
    assertTrue(Character.isTitleCase('\u1FA9'))
    assertTrue(Character.isTitleCase('\u1FAA'))
    assertTrue(Character.isTitleCase('\u1FAB'))
    assertTrue(Character.isTitleCase('\u1FAC'))
    assertTrue(Character.isTitleCase('\u1FAD'))
    assertTrue(Character.isTitleCase('\u1FAE'))
    assertTrue(Character.isTitleCase('\u1FAF'))
    assertTrue(Character.isTitleCase('\u1FBC'))
    assertTrue(Character.isTitleCase('\u1FCC'))
    assertTrue(Character.isTitleCase('\u1FFC'))

    // 100 randomly chosen characters that produce false
    assertFalse(Character.isTitleCase('\u043A'))
    assertFalse(Character.isTitleCase('\u09F9'))
    assertFalse(Character.isTitleCase('\u0BD0'))
    assertFalse(Character.isTitleCase('\u0DF3'))
    assertFalse(Character.isTitleCase('\u1290'))
    assertFalse(Character.isTitleCase('\u1A35'))
    assertFalse(Character.isTitleCase('\u1B86'))
    assertFalse(Character.isTitleCase('\u1DB3'))
    assertFalse(Character.isTitleCase('\u228B'))
    assertFalse(Character.isTitleCase('\u22D1'))
    assertFalse(Character.isTitleCase('\u231A'))
    assertFalse(Character.isTitleCase('\u2A69'))
    assertFalse(Character.isTitleCase('\u2AD3'))
    assertFalse(Character.isTitleCase('\u2B75'))
    assertFalse(Character.isTitleCase('\u2FDC'))
    assertFalse(Character.isTitleCase('\u323F'))
    assertFalse(Character.isTitleCase('\u3354'))
    assertFalse(Character.isTitleCase('\u35AA'))
    assertFalse(Character.isTitleCase('\u3E32'))
    assertFalse(Character.isTitleCase('\u3EF6'))
    assertFalse(Character.isTitleCase('\u4251'))
    assertFalse(Character.isTitleCase('\u4860'))
    assertFalse(Character.isTitleCase('\u4CF7'))
    assertFalse(Character.isTitleCase('\u50C7'))
    assertFalse(Character.isTitleCase('\u531E'))
    assertFalse(Character.isTitleCase('\u54E9'))
    assertFalse(Character.isTitleCase('\u57F4'))
    assertFalse(Character.isTitleCase('\u5870'))
    assertFalse(Character.isTitleCase('\u58BF'))
    assertFalse(Character.isTitleCase('\u599B'))
    assertFalse(Character.isTitleCase('\u5D43'))
    assertFalse(Character.isTitleCase('\u62DC'))
    assertFalse(Character.isTitleCase('\u6BCA'))
    assertFalse(Character.isTitleCase('\u6DB8'))
    assertFalse(Character.isTitleCase('\u711E'))
    assertFalse(Character.isTitleCase('\u75E4'))
    assertFalse(Character.isTitleCase('\u7926'))
    assertFalse(Character.isTitleCase('\u7BBD'))
    assertFalse(Character.isTitleCase('\u7C46'))
    assertFalse(Character.isTitleCase('\u7CA5'))
    assertFalse(Character.isTitleCase('\u7E88'))
    assertFalse(Character.isTitleCase('\u7FA9'))
    assertFalse(Character.isTitleCase('\u817A'))
    assertFalse(Character.isTitleCase('\u82BD'))
    assertFalse(Character.isTitleCase('\u82F0'))
    assertFalse(Character.isTitleCase('\u8435'))
    assertFalse(Character.isTitleCase('\u85A5'))
    assertFalse(Character.isTitleCase('\u9080'))
    assertFalse(Character.isTitleCase('\u93BF'))
    assertFalse(Character.isTitleCase('\u9A88'))
    assertFalse(Character.isTitleCase('\u9AF9'))
    assertFalse(Character.isTitleCase('\uA18C'))
    assertFalse(Character.isTitleCase('\uA5C4'))
    assertFalse(Character.isTitleCase('\uA91B'))
    assertFalse(Character.isTitleCase('\uAD45'))
    assertFalse(Character.isTitleCase('\uAE88'))
    assertFalse(Character.isTitleCase('\uB06F'))
    assertFalse(Character.isTitleCase('\uB2E0'))
    assertFalse(Character.isTitleCase('\uB427'))
    assertFalse(Character.isTitleCase('\uB512'))
    assertFalse(Character.isTitleCase('\uB59C'))
    assertFalse(Character.isTitleCase('\uB79B'))
    assertFalse(Character.isTitleCase('\uB911'))
    assertFalse(Character.isTitleCase('\uB995'))
    assertFalse(Character.isTitleCase('\uBA25'))
    assertFalse(Character.isTitleCase('\uBFED'))
    assertFalse(Character.isTitleCase('\uC0E1'))
    assertFalse(Character.isTitleCase('\uC5CC'))
    assertFalse(Character.isTitleCase('\uC80B'))
    assertFalse(Character.isTitleCase('\uC823'))
    assertFalse(Character.isTitleCase('\uCAE2'))
    assertFalse(Character.isTitleCase('\uCC2E'))
    assertFalse(Character.isTitleCase('\uCD96'))
    assertFalse(Character.isTitleCase('\uD025'))
    assertFalse(Character.isTitleCase('\uD270'))
    assertFalse(Character.isTitleCase('\uD3B2'))
    assertFalse(Character.isTitleCase('\uD679'))
    assertFalse(Character.isTitleCase('\uD822'))
    assertFalse(Character.isTitleCase('\uDA58'))
    assertFalse(Character.isTitleCase('\uE1FE'))
    assertFalse(Character.isTitleCase('\uE247'))
    assertFalse(Character.isTitleCase('\uE41E'))
    assertFalse(Character.isTitleCase('\uE518'))
    assertFalse(Character.isTitleCase('\uE6F1'))
    assertFalse(Character.isTitleCase('\uE7E8'))
    assertFalse(Character.isTitleCase('\uE89F'))
    assertFalse(Character.isTitleCase('\uEA41'))
    assertFalse(Character.isTitleCase('\uEA6D'))
    assertFalse(Character.isTitleCase('\uEAB6'))
    assertFalse(Character.isTitleCase('\uEBD4'))
    assertFalse(Character.isTitleCase('\uEF15'))
    assertFalse(Character.isTitleCase('\uF506'))
    assertFalse(Character.isTitleCase('\uF519'))
    assertFalse(Character.isTitleCase('\uF53F'))
    assertFalse(Character.isTitleCase('\uF5EC'))
    assertFalse(Character.isTitleCase('\uF62F'))
    assertFalse(Character.isTitleCase('\uFAD4'))
    assertFalse(Character.isTitleCase('\uFDBC'))
    assertFalse(Character.isTitleCase('\u0029'))
    assertFalse(Character.isTitleCase('\u0070'))
  }

  @Test def isMirrored(): Unit = {
    // 50 randomly chosen characters that produce true
    assertTrue(Character.isMirrored('\u208D'))
    assertTrue(Character.isMirrored('\u208E'))
    assertTrue(Character.isMirrored('\u2140'))
    assertTrue(Character.isMirrored('\u2201'))
    assertTrue(Character.isMirrored('\u2204'))
    assertTrue(Character.isMirrored('\u221A'))
    assertTrue(Character.isMirrored('\u221B'))
    assertTrue(Character.isMirrored('\u221D'))
    assertTrue(Character.isMirrored('\u222D'))
    assertTrue(Character.isMirrored('\u2232'))
    assertTrue(Character.isMirrored('\u2252'))
    assertTrue(Character.isMirrored('\u2254'))
    assertTrue(Character.isMirrored('\u226A'))
    assertTrue(Character.isMirrored('\u2271'))
    assertTrue(Character.isMirrored('\u227A'))
    assertTrue(Character.isMirrored('\u2281'))
    assertTrue(Character.isMirrored('\u2284'))
    assertTrue(Character.isMirrored('\u2286'))
    assertTrue(Character.isMirrored('\u22A2'))
    assertTrue(Character.isMirrored('\u22D0'))
    assertTrue(Character.isMirrored('\u22DA'))
    assertTrue(Character.isMirrored('\u22F1'))
    assertTrue(Character.isMirrored('\u22F2'))
    assertTrue(Character.isMirrored('\u22F9'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isMirrored('\u27C3'))
    if (!executingInJVMOnJDK7OrLower)
      assertTrue(Character.isMirrored('\u27CD'))
    assertTrue(Character.isMirrored('\u27EA'))
    assertTrue(Character.isMirrored('\u2988'))
    assertTrue(Character.isMirrored('\u298D'))
    assertTrue(Character.isMirrored('\u299C'))
    assertTrue(Character.isMirrored('\u29A5'))
    assertTrue(Character.isMirrored('\u29D1'))
    assertTrue(Character.isMirrored('\u29D9'))
    assertTrue(Character.isMirrored('\u29E4'))
    assertTrue(Character.isMirrored('\u29F5'))
    assertTrue(Character.isMirrored('\u2A1C'))
    assertTrue(Character.isMirrored('\u2A89'))
    assertTrue(Character.isMirrored('\u2A94'))
    assertTrue(Character.isMirrored('\u2AAB'))
    assertTrue(Character.isMirrored('\u2ABE'))
    assertTrue(Character.isMirrored('\u2AF9'))
    assertTrue(Character.isMirrored('\u2AFD'))
    if (!executingInJVMOnJDK6) {
      assertTrue(Character.isMirrored('\u2E02'))
      assertTrue(Character.isMirrored('\u2E23'))
      assertTrue(Character.isMirrored('\u2E24'))
      assertTrue(Character.isMirrored('\uFE5C'))
      assertTrue(Character.isMirrored('\uFE64'))
    }

    // 50 randomly chosen characters that produce false
    assertFalse(Character.isMirrored('\u0688'))
    assertFalse(Character.isMirrored('\u9319'))
    assertFalse(Character.isMirrored('\uFAF5'))
    assertFalse(Character.isMirrored(1030807))
    assertFalse(Character.isMirrored(1032997))
    assertFalse(Character.isMirrored(1073298))
    assertFalse(Character.isMirrored(1087805))
    assertFalse(Character.isMirrored(1090056))
    assertFalse(Character.isMirrored(1106453))
    assertFalse(Character.isMirrored(112391))
    assertFalse(Character.isMirrored(114041))
    assertFalse(Character.isMirrored(135408))
    assertFalse(Character.isMirrored(139769))
    assertFalse(Character.isMirrored(141057))
    assertFalse(Character.isMirrored(141146))
    assertFalse(Character.isMirrored(214605))
    assertFalse(Character.isMirrored(245103))
    assertFalse(Character.isMirrored(245550))
    assertFalse(Character.isMirrored(265448))
    assertFalse(Character.isMirrored(289372))
    assertFalse(Character.isMirrored(290850))
    assertFalse(Character.isMirrored(314422))
    assertFalse(Character.isMirrored(360533))
    assertFalse(Character.isMirrored(361613))
    assertFalse(Character.isMirrored(399166))
    assertFalse(Character.isMirrored(418562))
    assertFalse(Character.isMirrored(441100))
    assertFalse(Character.isMirrored(466467))
    assertFalse(Character.isMirrored(478964))
    assertFalse(Character.isMirrored(528293))
    assertFalse(Character.isMirrored(533681))
    assertFalse(Character.isMirrored(557036))
    assertFalse(Character.isMirrored(599773))
    assertFalse(Character.isMirrored(666210))
    assertFalse(Character.isMirrored(67175))
    assertFalse(Character.isMirrored(673481))
    assertFalse(Character.isMirrored(714304))
    assertFalse(Character.isMirrored(725335))
    assertFalse(Character.isMirrored(746217))
    assertFalse(Character.isMirrored(74649))
    assertFalse(Character.isMirrored(777837))
    assertFalse(Character.isMirrored(808419))
    assertFalse(Character.isMirrored(829990))
    assertFalse(Character.isMirrored(83437))
    assertFalse(Character.isMirrored(907564))
    assertFalse(Character.isMirrored(918918))
    assertFalse(Character.isMirrored(921306))
    assertFalse(Character.isMirrored(925902))
    assertFalse(Character.isMirrored(927621))
    assertFalse(Character.isMirrored(962242))
  }

  @Test def isDefined(): Unit = {
    // 100 randomly chosen characters that produce true
    assertTrue(Character.isDefined('\u0191'))
    assertTrue(Character.isDefined('\u03D6'))
    assertTrue(Character.isDefined('\u04A1'))
    assertTrue(Character.isDefined('\u04BD'))
    assertTrue(Character.isDefined('\u07AC'))
    assertTrue(Character.isDefined('\u0BED'))
    assertTrue(Character.isDefined('\u0D21'))
    assertTrue(Character.isDefined('\u0D6B'))
    if (!executingInJVMOnJDK6) {
      assertTrue(Character.isDefined('\u106D'))
      assertTrue(Character.isDefined('\u1982'))
      assertTrue(Character.isDefined('\u1A46'))
      assertTrue(Character.isDefined('\u1CD5'))
    }
    assertTrue(Character.isDefined('\u1E87'))
    assertTrue(Character.isDefined('\u206C'))
    assertTrue(Character.isDefined('\u22F6'))
    assertTrue(Character.isDefined('\u2325'))
    assertTrue(Character.isDefined('\u277C'))
    if (!executingInJVMOnJDK6) {
      assertTrue(Character.isDefined('\u2E0D'))
      assertTrue(Character.isDefined('\u2E27'))
    }
    assertTrue(Character.isDefined('\u2FA4'))
    assertTrue(Character.isDefined('\u3163'))
    assertTrue(Character.isDefined('\u32D7'))
    assertTrue(Character.isDefined('\u3384'))
    assertTrue(Character.isDefined('\u33D3'))
    assertTrue(Character.isDefined('\u35D9'))
    assertTrue(Character.isDefined('\u36F2'))
    assertTrue(Character.isDefined('\u385B'))
    assertTrue(Character.isDefined('\u398D'))
    assertTrue(Character.isDefined('\u3A81'))
    assertTrue(Character.isDefined('\u3D57'))
    assertTrue(Character.isDefined('\u3DD4'))
    assertTrue(Character.isDefined('\u415C'))
    assertTrue(Character.isDefined('\u4233'))
    assertTrue(Character.isDefined('\u4A87'))
    assertTrue(Character.isDefined('\u50C7'))
    assertTrue(Character.isDefined('\u51A2'))
    assertTrue(Character.isDefined('\u525B'))
    assertTrue(Character.isDefined('\u62D0'))
    assertTrue(Character.isDefined('\u6405'))
    assertTrue(Character.isDefined('\u67D2'))
    assertTrue(Character.isDefined('\u6ED4'))
    assertTrue(Character.isDefined('\u6F13'))
    assertTrue(Character.isDefined('\u7366'))
    assertTrue(Character.isDefined('\u7661'))
    assertTrue(Character.isDefined('\u76FB'))
    assertTrue(Character.isDefined('\u77D7'))
    assertTrue(Character.isDefined('\u7A26'))
    assertTrue(Character.isDefined('\u7E41'))
    assertTrue(Character.isDefined('\u7E9E'))
    assertTrue(Character.isDefined('\u7F5E'))
    assertTrue(Character.isDefined('\u8336'))
    assertTrue(Character.isDefined('\u842D'))
    assertTrue(Character.isDefined('\u8488'))
    assertTrue(Character.isDefined('\u88C9'))
    assertTrue(Character.isDefined('\u8A81'))
    assertTrue(Character.isDefined('\u8D21'))
    assertTrue(Character.isDefined('\u8EA0'))
    assertTrue(Character.isDefined('\u921B'))
    assertTrue(Character.isDefined('\u94EC'))
    assertTrue(Character.isDefined('\u972C'))
    assertTrue(Character.isDefined('\u98D4'))
    assertTrue(Character.isDefined('\u993D'))
    assertTrue(Character.isDefined('\u9B13'))
    assertTrue(Character.isDefined('\u9B6F'))
    assertTrue(Character.isDefined('\uA278'))
    assertTrue(Character.isDefined('\uA2F9'))
    if (!executingInJVMOnJDK6) {
      assertTrue(Character.isDefined('\uA618'))
      assertTrue(Character.isDefined('\uA789'))
      assertTrue(Character.isDefined('\uAA1C'))
    }
    assertTrue(Character.isDefined('\uAEF8'))
    assertTrue(Character.isDefined('\uAF74'))
    assertTrue(Character.isDefined('\uB170'))
    assertTrue(Character.isDefined('\uB508'))
    assertTrue(Character.isDefined('\uB820'))
    assertTrue(Character.isDefined('\uB8E3'))
    assertTrue(Character.isDefined('\uBC34'))
    assertTrue(Character.isDefined('\uC501'))
    assertTrue(Character.isDefined('\uCABF'))
    assertTrue(Character.isDefined('\uCC21'))
    assertTrue(Character.isDefined('\uCC27'))
    assertTrue(Character.isDefined('\uCC46'))
    assertTrue(Character.isDefined('\uCC7F'))
    assertTrue(Character.isDefined('\uCD32'))
    assertTrue(Character.isDefined('\uCFE0'))
    assertTrue(Character.isDefined('\uD08F'))
    assertTrue(Character.isDefined('\uD4AF'))
    assertTrue(Character.isDefined('\uD522'))
    assertTrue(Character.isDefined('\uD9F4'))
    assertTrue(Character.isDefined('\uE5A7'))
    assertTrue(Character.isDefined('\uEB96'))
    assertTrue(Character.isDefined('\uEEC0'))
    assertTrue(Character.isDefined('\uEF30'))
    assertTrue(Character.isDefined('\uEFD9'))
    assertTrue(Character.isDefined('\uF4E8'))
    assertTrue(Character.isDefined('\uF547'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isDefined('\uFA7D'))
    assertTrue(Character.isDefined('\uFD39'))
    assertTrue(Character.isDefined('\uFF3A'))

    // 100 randomly chosen characters that produce false
    assertFalse(Character.isDefined('\u0528'))
    assertFalse(Character.isDefined('\u052B'))
    assertFalse(Character.isDefined('\u058D'))
    assertFalse(Character.isDefined('\u05FE'))
    assertFalse(Character.isDefined('\u07BE'))
    assertFalse(Character.isDefined('\u07FB'))
    assertFalse(Character.isDefined('\u0878'))
    assertFalse(Character.isDefined('\u0879'))
    assertFalse(Character.isDefined('\u088E'))
    assertFalse(Character.isDefined('\u0892'))
    assertFalse(Character.isDefined('\u08BE'))
    assertFalse(Character.isDefined('\u08C5'))
    assertFalse(Character.isDefined('\u08D4'))
    assertFalse(Character.isDefined('\u08DB'))
    assertFalse(Character.isDefined('\u08E1'))
    assertFalse(Character.isDefined('\u09B1'))
    assertFalse(Character.isDefined('\u09D0'))
    assertFalse(Character.isDefined('\u09D5'))
    assertFalse(Character.isDefined('\u09FF'))
    assertFalse(Character.isDefined('\u0A12'))
    assertFalse(Character.isDefined('\u0A52'))
    assertFalse(Character.isDefined('\u0A60'))
    assertFalse(Character.isDefined('\u0AE4'))
    assertFalse(Character.isDefined('\u0AF4'))
    assertFalse(Character.isDefined('\u0B3B'))
    assertFalse(Character.isDefined('\u0B7F'))
    assertFalse(Character.isDefined('\u0BBA'))
    assertFalse(Character.isDefined('\u0BBD'))
    assertFalse(Character.isDefined('\u0CDF'))
    assertFalse(Character.isDefined('\u0D49'))
    assertFalse(Character.isDefined('\u0D99'))
    assertFalse(Character.isDefined('\u0DBC'))
    assertFalse(Character.isDefined('\u0E00'))
    assertFalse(Character.isDefined('\u0E74'))
    assertFalse(Character.isDefined('\u0E8E'))
    assertFalse(Character.isDefined('\u0E93'))
    assertFalse(Character.isDefined('\u0EA8'))
    assertFalse(Character.isDefined('\u0EF6'))
    assertFalse(Character.isDefined('\u0FDB'))
    assertFalse(Character.isDefined('\u0FE5'))
    assertFalse(Character.isDefined('\u10C9'))
    assertFalse(Character.isDefined('\u12C6'))
    assertFalse(Character.isDefined('\u16F7'))
    assertFalse(Character.isDefined('\u16FA'))
    assertFalse(Character.isDefined('\u1739'))
    assertFalse(Character.isDefined('\u173A'))
    assertFalse(Character.isDefined('\u177A'))
    assertFalse(Character.isDefined('\u17DF'))
    assertFalse(Character.isDefined('\u187A'))
    assertFalse(Character.isDefined('\u1C39'))
    assertFalse(Character.isDefined('\u1CAC'))
    assertFalse(Character.isDefined('\u1CAF'))
    assertFalse(Character.isDefined('\u1CBB'))
    assertFalse(Character.isDefined('\u1CC9'))
    assertFalse(Character.isDefined('\u1FF1'))
    assertFalse(Character.isDefined('\u2073'))
    assertFalse(Character.isDefined('\u23F7'))
    assertFalse(Character.isDefined('\u23F8'))
    assertFalse(Character.isDefined('\u243F'))
    assertFalse(Character.isDefined('\u2700'))
    assertFalse(Character.isDefined('\u2B63'))
    assertFalse(Character.isDefined('\u2B7F'))
    assertFalse(Character.isDefined('\u2BDC'))
    assertFalse(Character.isDefined('\u2BF5'))
    assertFalse(Character.isDefined('\u2BFE'))
    assertFalse(Character.isDefined('\u2C2F'))
    assertFalse(Character.isDefined('\u2EFE'))
    assertFalse(Character.isDefined('\u2FE9'))
    assertFalse(Character.isDefined('\u2FEE'))
    assertFalse(Character.isDefined('\u31EE'))
    assertFalse(Character.isDefined('\u32FF'))
    assertFalse(Character.isDefined('\u9FE9'))
    assertFalse(Character.isDefined('\uA639'))
    assertFalse(Character.isDefined('\uA7AE'))
    assertFalse(Character.isDefined('\uA7EC'))
    assertFalse(Character.isDefined('\uA7F2'))
    assertFalse(Character.isDefined('\uA8C7'))
    assertFalse(Character.isDefined('\uA8DD'))
    assertFalse(Character.isDefined('\uA8DF'))
    assertFalse(Character.isDefined('\uA957'))
    assertFalse(Character.isDefined('\uAAC6'))
    assertFalse(Character.isDefined('\uAAC9'))
    assertFalse(Character.isDefined('\uAAD9'))
    assertFalse(Character.isDefined('\uAAF9'))
    assertFalse(Character.isDefined('\uAB46'))
    assertFalse(Character.isDefined('\uAB5B'))
    assertFalse(Character.isDefined('\uAB70'))
    assertFalse(Character.isDefined('\uAB95'))
    assertFalse(Character.isDefined('\uABA7'))
    assertFalse(Character.isDefined('\uD7C9'))
    assertFalse(Character.isDefined('\uFAE1'))
    assertFalse(Character.isDefined('\uFAE9'))
    assertFalse(Character.isDefined('\uFB12'))
    assertFalse(Character.isDefined('\uFBCA'))
    assertFalse(Character.isDefined('\uFD40'))
    assertFalse(Character.isDefined('\uFFF3'))
  }

  @Test def getType(): Unit = {
    // 300 first characters
    assertEquals(15, Character.getType(0))
    assertEquals(15, Character.getType(1))
    assertEquals(15, Character.getType(2))
    assertEquals(15, Character.getType(3))
    assertEquals(15, Character.getType(4))
    assertEquals(15, Character.getType(5))
    assertEquals(15, Character.getType(6))
    assertEquals(15, Character.getType(7))
    assertEquals(15, Character.getType(8))
    assertEquals(15, Character.getType(9))
    assertEquals(15, Character.getType(10))
    assertEquals(15, Character.getType(11))
    assertEquals(15, Character.getType(12))
    assertEquals(15, Character.getType(13))
    assertEquals(15, Character.getType(14))
    assertEquals(15, Character.getType(15))
    assertEquals(15, Character.getType(16))
    assertEquals(15, Character.getType(17))
    assertEquals(15, Character.getType(18))
    assertEquals(15, Character.getType(19))
    assertEquals(15, Character.getType(20))
    assertEquals(15, Character.getType(21))
    assertEquals(15, Character.getType(22))
    assertEquals(15, Character.getType(23))
    assertEquals(15, Character.getType(24))
    assertEquals(15, Character.getType(25))
    assertEquals(15, Character.getType(26))
    assertEquals(15, Character.getType(27))
    assertEquals(15, Character.getType(28))
    assertEquals(15, Character.getType(29))
    assertEquals(15, Character.getType(30))
    assertEquals(15, Character.getType(31))
    assertEquals(12, Character.getType(32))
    assertEquals(24, Character.getType(33))
    assertEquals(24, Character.getType(34))
    assertEquals(24, Character.getType(35))
    assertEquals(26, Character.getType(36))
    assertEquals(24, Character.getType(37))
    assertEquals(24, Character.getType(38))
    assertEquals(24, Character.getType(39))
    assertEquals(21, Character.getType(40))
    assertEquals(22, Character.getType(41))
    assertEquals(24, Character.getType(42))
    assertEquals(25, Character.getType(43))
    assertEquals(24, Character.getType(44))
    assertEquals(20, Character.getType(45))
    assertEquals(24, Character.getType(46))
    assertEquals(24, Character.getType(47))
    assertEquals(9, Character.getType(48))
    assertEquals(9, Character.getType(49))
    assertEquals(9, Character.getType(50))
    assertEquals(9, Character.getType(51))
    assertEquals(9, Character.getType(52))
    assertEquals(9, Character.getType(53))
    assertEquals(9, Character.getType(54))
    assertEquals(9, Character.getType(55))
    assertEquals(9, Character.getType(56))
    assertEquals(9, Character.getType(57))
    assertEquals(24, Character.getType(58))
    assertEquals(24, Character.getType(59))
    assertEquals(25, Character.getType(60))
    assertEquals(25, Character.getType(61))
    assertEquals(25, Character.getType(62))
    assertEquals(24, Character.getType(63))
    assertEquals(24, Character.getType(64))
    assertEquals(1, Character.getType(65))
    assertEquals(1, Character.getType(66))
    assertEquals(1, Character.getType(67))
    assertEquals(1, Character.getType(68))
    assertEquals(1, Character.getType(69))
    assertEquals(1, Character.getType(70))
    assertEquals(1, Character.getType(71))
    assertEquals(1, Character.getType(72))
    assertEquals(1, Character.getType(73))
    assertEquals(1, Character.getType(74))
    assertEquals(1, Character.getType(75))
    assertEquals(1, Character.getType(76))
    assertEquals(1, Character.getType(77))
    assertEquals(1, Character.getType(78))
    assertEquals(1, Character.getType(79))
    assertEquals(1, Character.getType(80))
    assertEquals(1, Character.getType(81))
    assertEquals(1, Character.getType(82))
    assertEquals(1, Character.getType(83))
    assertEquals(1, Character.getType(84))
    assertEquals(1, Character.getType(85))
    assertEquals(1, Character.getType(86))
    assertEquals(1, Character.getType(87))
    assertEquals(1, Character.getType(88))
    assertEquals(1, Character.getType(89))
    assertEquals(1, Character.getType(90))
    assertEquals(21, Character.getType(91))
    assertEquals(24, Character.getType(92))
    assertEquals(22, Character.getType(93))
    assertEquals(27, Character.getType(94))
    assertEquals(23, Character.getType(95))
    assertEquals(27, Character.getType(96))
    assertEquals(2, Character.getType(97))
    assertEquals(2, Character.getType(98))
    assertEquals(2, Character.getType(99))
    assertEquals(2, Character.getType(100))
    assertEquals(2, Character.getType(101))
    assertEquals(2, Character.getType(102))
    assertEquals(2, Character.getType(103))
    assertEquals(2, Character.getType(104))
    assertEquals(2, Character.getType(105))
    assertEquals(2, Character.getType(106))
    assertEquals(2, Character.getType(107))
    assertEquals(2, Character.getType(108))
    assertEquals(2, Character.getType(109))
    assertEquals(2, Character.getType(110))
    assertEquals(2, Character.getType(111))
    assertEquals(2, Character.getType(112))
    assertEquals(2, Character.getType(113))
    assertEquals(2, Character.getType(114))
    assertEquals(2, Character.getType(115))
    assertEquals(2, Character.getType(116))
    assertEquals(2, Character.getType(117))
    assertEquals(2, Character.getType(118))
    assertEquals(2, Character.getType(119))
    assertEquals(2, Character.getType(120))
    assertEquals(2, Character.getType(121))
    assertEquals(2, Character.getType(122))
    assertEquals(21, Character.getType(123))
    assertEquals(25, Character.getType(124))
    assertEquals(22, Character.getType(125))
    assertEquals(25, Character.getType(126))
    assertEquals(15, Character.getType(127))
    assertEquals(15, Character.getType(128))
    assertEquals(15, Character.getType(129))
    assertEquals(15, Character.getType(130))
    assertEquals(15, Character.getType(131))
    assertEquals(15, Character.getType(132))
    assertEquals(15, Character.getType(133))
    assertEquals(15, Character.getType(134))
    assertEquals(15, Character.getType(135))
    assertEquals(15, Character.getType(136))
    assertEquals(15, Character.getType(137))
    assertEquals(15, Character.getType(138))
    assertEquals(15, Character.getType(139))
    assertEquals(15, Character.getType(140))
    assertEquals(15, Character.getType(141))
    assertEquals(15, Character.getType(142))
    assertEquals(15, Character.getType(143))
    assertEquals(15, Character.getType(144))
    assertEquals(15, Character.getType(145))
    assertEquals(15, Character.getType(146))
    assertEquals(15, Character.getType(147))
    assertEquals(15, Character.getType(148))
    assertEquals(15, Character.getType(149))
    assertEquals(15, Character.getType(150))
    assertEquals(15, Character.getType(151))
    assertEquals(15, Character.getType(152))
    assertEquals(15, Character.getType(153))
    assertEquals(15, Character.getType(154))
    assertEquals(15, Character.getType(155))
    assertEquals(15, Character.getType(156))
    assertEquals(15, Character.getType(157))
    assertEquals(15, Character.getType(158))
    assertEquals(15, Character.getType(159))
    assertEquals(12, Character.getType(160))
    assertEquals(24, Character.getType(161))
    assertEquals(26, Character.getType(162))
    assertEquals(26, Character.getType(163))
    assertEquals(26, Character.getType(164))
    assertEquals(26, Character.getType(165))
    assertEquals(28, Character.getType(166))
    if (!executingInJVMOnJDK7OrLower)
      assertEquals(24, Character.getType(167))
    assertEquals(27, Character.getType(168))
    assertEquals(28, Character.getType(169))
    if (!executingInJVMOnJDK7OrLower)
      assertEquals(5, Character.getType(170))
    assertEquals(29, Character.getType(171))
    assertEquals(25, Character.getType(172))
    assertEquals(16, Character.getType(173))
    assertEquals(28, Character.getType(174))
    assertEquals(27, Character.getType(175))
    assertEquals(28, Character.getType(176))
    assertEquals(25, Character.getType(177))
    assertEquals(11, Character.getType(178))
    assertEquals(11, Character.getType(179))
    assertEquals(27, Character.getType(180))
    assertEquals(2, Character.getType(181))
    if (!executingInJVMOnJDK7OrLower)
      assertEquals(24, Character.getType(182))
    assertEquals(24, Character.getType(183))
    assertEquals(27, Character.getType(184))
    assertEquals(11, Character.getType(185))
    if (!executingInJVMOnJDK7OrLower)
      assertEquals(5, Character.getType(186))
    assertEquals(30, Character.getType(187))
    assertEquals(11, Character.getType(188))
    assertEquals(11, Character.getType(189))
    assertEquals(11, Character.getType(190))
    assertEquals(24, Character.getType(191))
    assertEquals(1, Character.getType(192))
    assertEquals(1, Character.getType(193))
    assertEquals(1, Character.getType(194))
    assertEquals(1, Character.getType(195))
    assertEquals(1, Character.getType(196))
    assertEquals(1, Character.getType(197))
    assertEquals(1, Character.getType(198))
    assertEquals(1, Character.getType(199))
    assertEquals(1, Character.getType(200))
    assertEquals(1, Character.getType(201))
    assertEquals(1, Character.getType(202))
    assertEquals(1, Character.getType(203))
    assertEquals(1, Character.getType(204))
    assertEquals(1, Character.getType(205))
    assertEquals(1, Character.getType(206))
    assertEquals(1, Character.getType(207))
    assertEquals(1, Character.getType(208))
    assertEquals(1, Character.getType(209))
    assertEquals(1, Character.getType(210))
    assertEquals(1, Character.getType(211))
    assertEquals(1, Character.getType(212))
    assertEquals(1, Character.getType(213))
    assertEquals(1, Character.getType(214))
    assertEquals(25, Character.getType(215))
    assertEquals(1, Character.getType(216))
    assertEquals(1, Character.getType(217))
    assertEquals(1, Character.getType(218))
    assertEquals(1, Character.getType(219))
    assertEquals(1, Character.getType(220))
    assertEquals(1, Character.getType(221))
    assertEquals(1, Character.getType(222))
    assertEquals(2, Character.getType(223))
    assertEquals(2, Character.getType(224))
    assertEquals(2, Character.getType(225))
    assertEquals(2, Character.getType(226))
    assertEquals(2, Character.getType(227))
    assertEquals(2, Character.getType(228))
    assertEquals(2, Character.getType(229))
    assertEquals(2, Character.getType(230))
    assertEquals(2, Character.getType(231))
    assertEquals(2, Character.getType(232))
    assertEquals(2, Character.getType(233))
    assertEquals(2, Character.getType(234))
    assertEquals(2, Character.getType(235))
    assertEquals(2, Character.getType(236))
    assertEquals(2, Character.getType(237))
    assertEquals(2, Character.getType(238))
    assertEquals(2, Character.getType(239))
    assertEquals(2, Character.getType(240))
    assertEquals(2, Character.getType(241))
    assertEquals(2, Character.getType(242))
    assertEquals(2, Character.getType(243))
    assertEquals(2, Character.getType(244))
    assertEquals(2, Character.getType(245))
    assertEquals(2, Character.getType(246))
    assertEquals(25, Character.getType(247))
    assertEquals(2, Character.getType(248))
    assertEquals(2, Character.getType(249))
    assertEquals(2, Character.getType(250))
    assertEquals(2, Character.getType(251))
    assertEquals(2, Character.getType(252))
    assertEquals(2, Character.getType(253))
    assertEquals(2, Character.getType(254))
    assertEquals(2, Character.getType(255))
    assertEquals(1, Character.getType(256))
    assertEquals(2, Character.getType(257))
    assertEquals(1, Character.getType(258))
    assertEquals(2, Character.getType(259))
    assertEquals(1, Character.getType(260))
    assertEquals(2, Character.getType(261))
    assertEquals(1, Character.getType(262))
    assertEquals(2, Character.getType(263))
    assertEquals(1, Character.getType(264))
    assertEquals(2, Character.getType(265))
    assertEquals(1, Character.getType(266))
    assertEquals(2, Character.getType(267))
    assertEquals(1, Character.getType(268))
    assertEquals(2, Character.getType(269))
    assertEquals(1, Character.getType(270))
    assertEquals(2, Character.getType(271))
    assertEquals(1, Character.getType(272))
    assertEquals(2, Character.getType(273))
    assertEquals(1, Character.getType(274))
    assertEquals(2, Character.getType(275))
    assertEquals(1, Character.getType(276))
    assertEquals(2, Character.getType(277))
    assertEquals(1, Character.getType(278))
    assertEquals(2, Character.getType(279))
    assertEquals(1, Character.getType(280))
    assertEquals(2, Character.getType(281))
    assertEquals(1, Character.getType(282))
    assertEquals(2, Character.getType(283))
    assertEquals(1, Character.getType(284))
    assertEquals(2, Character.getType(285))
    assertEquals(1, Character.getType(286))
    assertEquals(2, Character.getType(287))
    assertEquals(1, Character.getType(288))
    assertEquals(2, Character.getType(289))
    assertEquals(1, Character.getType(290))
    assertEquals(2, Character.getType(291))
    assertEquals(1, Character.getType(292))
    assertEquals(2, Character.getType(293))
    assertEquals(1, Character.getType(294))
    assertEquals(2, Character.getType(295))
    assertEquals(1, Character.getType(296))
    assertEquals(2, Character.getType(297))
    assertEquals(1, Character.getType(298))
    assertEquals(2, Character.getType(299))

    // 400 randomly chosen characters
    assertEquals(5, Character.getType(2581))
    assertEquals(28, Character.getType(9237))
    assertEquals(25, Character.getType(10583))
    assertEquals(5, Character.getType(14526))
    assertEquals(5, Character.getType(16468))
    assertEquals(5, Character.getType(18538))
    assertEquals(5, Character.getType(21037))
    assertEquals(5, Character.getType(21880))
    assertEquals(5, Character.getType(26084))
    assertEquals(5, Character.getType(29876))
    assertEquals(5, Character.getType(31553))
    assertEquals(5, Character.getType(33192))
    assertEquals(5, Character.getType(34508))
    assertEquals(5, Character.getType(38994))
    assertEquals(5, Character.getType(41384))
    assertEquals(5, Character.getType(41661))
    assertEquals(0, Character.getType(43349))
    assertEquals(5, Character.getType(45859))
    assertEquals(5, Character.getType(46867))
    assertEquals(5, Character.getType(48796))
    assertEquals(5, Character.getType(50755))
    assertEquals(5, Character.getType(54077))
    assertEquals(19, Character.getType(55534))
    assertEquals(18, Character.getType(57625))
    assertEquals(18, Character.getType(59186))
    assertEquals(5, Character.getType(64622))
    assertEquals(0, Character.getType(67177))
    assertEquals(0, Character.getType(73067))
    if (!executingInJVMOnJDK6)
      assertEquals(5, Character.getType(78228))
    assertEquals(0, Character.getType(80656))
    assertEquals(0, Character.getType(81498))
    assertEquals(0, Character.getType(84617))
    assertEquals(0, Character.getType(86083))
    assertEquals(0, Character.getType(87255))
    assertEquals(0, Character.getType(88238))
    assertEquals(0, Character.getType(93812))
    assertEquals(0, Character.getType(95123))
    assertEquals(0, Character.getType(95326))
    assertEquals(0, Character.getType(96045))
    assertEquals(0, Character.getType(100120))
    assertEquals(0, Character.getType(103365))
    assertEquals(0, Character.getType(103707))
    assertEquals(0, Character.getType(104477))
    assertEquals(0, Character.getType(106587))
    assertEquals(0, Character.getType(115218))
    assertEquals(0, Character.getType(115801))
    assertEquals(0, Character.getType(118758))
    assertEquals(1, Character.getType(120024))
    assertEquals(0, Character.getType(121054))
    assertEquals(0, Character.getType(123903))
    assertEquals(0, Character.getType(126770))
    assertEquals(0, Character.getType(130632))
    assertEquals(5, Character.getType(131952))
    assertEquals(5, Character.getType(133129))
    assertEquals(5, Character.getType(135815))
    assertEquals(5, Character.getType(136969))
    assertEquals(5, Character.getType(138445))
    assertEquals(5, Character.getType(147888))
    assertEquals(5, Character.getType(152546))
    assertEquals(5, Character.getType(158177))
    assertEquals(5, Character.getType(163037))
    assertEquals(5, Character.getType(165823))
    assertEquals(5, Character.getType(169103))
    assertEquals(5, Character.getType(171304))
    assertEquals(5, Character.getType(172648))
    assertEquals(0, Character.getType(178250))
    assertEquals(0, Character.getType(180704))
    assertEquals(0, Character.getType(181598))
    assertEquals(0, Character.getType(186964))
    assertEquals(0, Character.getType(189826))
    assertEquals(0, Character.getType(198340))
    assertEquals(0, Character.getType(198519))
    assertEquals(0, Character.getType(198994))
    assertEquals(0, Character.getType(205472))
    assertEquals(0, Character.getType(207585))
    assertEquals(0, Character.getType(207958))
    assertEquals(0, Character.getType(208721))
    assertEquals(0, Character.getType(219582))
    assertEquals(0, Character.getType(220968))
    assertEquals(0, Character.getType(227170))
    assertEquals(0, Character.getType(227743))
    assertEquals(0, Character.getType(228515))
    assertEquals(0, Character.getType(229394))
    assertEquals(0, Character.getType(231801))
    assertEquals(0, Character.getType(232551))
    assertEquals(0, Character.getType(236092))
    assertEquals(0, Character.getType(238859))
    assertEquals(0, Character.getType(252799))
    assertEquals(0, Character.getType(255563))
    assertEquals(0, Character.getType(255895))
    assertEquals(0, Character.getType(259241))
    assertEquals(0, Character.getType(262761))
    assertEquals(0, Character.getType(267258))
    assertEquals(0, Character.getType(269354))
    assertEquals(0, Character.getType(269509))
    assertEquals(0, Character.getType(270726))
    assertEquals(0, Character.getType(271384))
    assertEquals(0, Character.getType(273382))
    assertEquals(0, Character.getType(276703))
    assertEquals(0, Character.getType(283121))
    assertEquals(0, Character.getType(291093))
    assertEquals(0, Character.getType(291551))
    assertEquals(0, Character.getType(293904))
    assertEquals(0, Character.getType(298484))
    assertEquals(0, Character.getType(301055))
    assertEquals(0, Character.getType(301379))
    assertEquals(0, Character.getType(304943))
    assertEquals(0, Character.getType(308860))
    assertEquals(0, Character.getType(309265))
    assertEquals(0, Character.getType(309466))
    assertEquals(0, Character.getType(317057))
    assertEquals(0, Character.getType(317814))
    assertEquals(0, Character.getType(317888))
    assertEquals(0, Character.getType(318020))
    assertEquals(0, Character.getType(319842))
    assertEquals(0, Character.getType(319936))
    assertEquals(0, Character.getType(323455))
    assertEquals(0, Character.getType(325009))
    assertEquals(0, Character.getType(326679))
    assertEquals(0, Character.getType(334555))
    assertEquals(0, Character.getType(334667))
    assertEquals(0, Character.getType(338205))
    assertEquals(0, Character.getType(344885))
    assertEquals(0, Character.getType(346752))
    assertEquals(0, Character.getType(353260))
    assertEquals(0, Character.getType(353840))
    assertEquals(0, Character.getType(360041))
    assertEquals(0, Character.getType(372920))
    assertEquals(0, Character.getType(373103))
    assertEquals(0, Character.getType(379356))
    assertEquals(0, Character.getType(379837))
    assertEquals(0, Character.getType(381734))
    assertEquals(0, Character.getType(383920))
    assertEquals(0, Character.getType(386450))
    assertEquals(0, Character.getType(393443))
    assertEquals(0, Character.getType(396435))
    assertEquals(0, Character.getType(400158))
    assertEquals(0, Character.getType(405132))
    assertEquals(0, Character.getType(406139))
    assertEquals(0, Character.getType(406817))
    assertEquals(0, Character.getType(410764))
    assertEquals(0, Character.getType(414453))
    assertEquals(0, Character.getType(419295))
    assertEquals(0, Character.getType(419492))
    assertEquals(0, Character.getType(420616))
    assertEquals(0, Character.getType(422854))
    assertEquals(0, Character.getType(423357))
    assertEquals(0, Character.getType(423620))
    assertEquals(0, Character.getType(423847))
    assertEquals(0, Character.getType(428863))
    assertEquals(0, Character.getType(429006))
    assertEquals(0, Character.getType(431107))
    assertEquals(0, Character.getType(432114))
    assertEquals(0, Character.getType(433251))
    assertEquals(0, Character.getType(435139))
    assertEquals(0, Character.getType(436750))
    assertEquals(0, Character.getType(442304))
    assertEquals(0, Character.getType(445467))
    assertEquals(0, Character.getType(449619))
    assertEquals(0, Character.getType(451649))
    assertEquals(0, Character.getType(452796))
    assertEquals(0, Character.getType(459845))
    assertEquals(0, Character.getType(462233))
    assertEquals(0, Character.getType(463465))
    assertEquals(0, Character.getType(465403))
    assertEquals(0, Character.getType(471112))
    assertEquals(0, Character.getType(471280))
    assertEquals(0, Character.getType(471634))
    assertEquals(0, Character.getType(473755))
    assertEquals(0, Character.getType(478248))
    assertEquals(0, Character.getType(480678))
    assertEquals(0, Character.getType(481366))
    assertEquals(0, Character.getType(484780))
    assertEquals(0, Character.getType(485524))
    assertEquals(0, Character.getType(486494))
    assertEquals(0, Character.getType(487220))
    assertEquals(0, Character.getType(489888))
    assertEquals(0, Character.getType(496358))
    assertEquals(0, Character.getType(497004))
    assertEquals(0, Character.getType(501635))
    assertEquals(0, Character.getType(501664))
    assertEquals(0, Character.getType(502672))
    assertEquals(0, Character.getType(504847))
    assertEquals(0, Character.getType(505791))
    assertEquals(0, Character.getType(506017))
    assertEquals(0, Character.getType(507855))
    assertEquals(0, Character.getType(509653))
    assertEquals(0, Character.getType(510983))
    assertEquals(0, Character.getType(514728))
    assertEquals(0, Character.getType(515825))
    assertEquals(0, Character.getType(516177))
    assertEquals(0, Character.getType(517858))
    assertEquals(0, Character.getType(521649))
    assertEquals(0, Character.getType(524928))
    assertEquals(0, Character.getType(525367))
    assertEquals(0, Character.getType(529235))
    assertEquals(0, Character.getType(530004))
    assertEquals(0, Character.getType(531799))
    assertEquals(0, Character.getType(534476))
    assertEquals(0, Character.getType(536386))
    assertEquals(0, Character.getType(536507))
    assertEquals(0, Character.getType(537192))
    assertEquals(0, Character.getType(541286))
    assertEquals(0, Character.getType(546959))
    assertEquals(0, Character.getType(554955))
    assertEquals(0, Character.getType(556317))
    assertEquals(0, Character.getType(562153))
    assertEquals(0, Character.getType(563963))
    assertEquals(0, Character.getType(564020))
    assertEquals(0, Character.getType(570471))
    assertEquals(0, Character.getType(574309))
    assertEquals(0, Character.getType(574487))
    assertEquals(0, Character.getType(574547))
    assertEquals(0, Character.getType(579326))
    assertEquals(0, Character.getType(579428))
    assertEquals(0, Character.getType(580567))
    assertEquals(0, Character.getType(588479))
    assertEquals(0, Character.getType(591039))
    assertEquals(0, Character.getType(593320))
    assertEquals(0, Character.getType(595035))
    assertEquals(0, Character.getType(596829))
    assertEquals(0, Character.getType(597596))
    assertEquals(0, Character.getType(600146))
    assertEquals(0, Character.getType(601789))
    assertEquals(0, Character.getType(603308))
    assertEquals(0, Character.getType(608296))
    assertEquals(0, Character.getType(612581))
    assertEquals(0, Character.getType(624001))
    assertEquals(0, Character.getType(626774))
    assertEquals(0, Character.getType(629729))
    assertEquals(0, Character.getType(635306))
    assertEquals(0, Character.getType(635481))
    assertEquals(0, Character.getType(636738))
    assertEquals(0, Character.getType(638187))
    assertEquals(0, Character.getType(639000))
    assertEquals(0, Character.getType(639171))
    assertEquals(0, Character.getType(639770))
    assertEquals(0, Character.getType(641295))
    assertEquals(0, Character.getType(641459))
    assertEquals(0, Character.getType(646699))
    assertEquals(0, Character.getType(653350))
    assertEquals(0, Character.getType(654242))
    assertEquals(0, Character.getType(658888))
    assertEquals(0, Character.getType(658937))
    assertEquals(0, Character.getType(662614))
    assertEquals(0, Character.getType(665575))
    assertEquals(0, Character.getType(666687))
    assertEquals(0, Character.getType(670456))
    assertEquals(0, Character.getType(673425))
    assertEquals(0, Character.getType(675808))
    assertEquals(0, Character.getType(676819))
    assertEquals(0, Character.getType(678369))
    assertEquals(0, Character.getType(679339))
    assertEquals(0, Character.getType(691315))
    assertEquals(0, Character.getType(692032))
    assertEquals(0, Character.getType(692992))
    assertEquals(0, Character.getType(700100))
    assertEquals(0, Character.getType(701010))
    assertEquals(0, Character.getType(701221))
    assertEquals(0, Character.getType(703413))
    assertEquals(0, Character.getType(705847))
    assertEquals(0, Character.getType(710671))
    assertEquals(0, Character.getType(719904))
    assertEquals(0, Character.getType(720047))
    assertEquals(0, Character.getType(721637))
    assertEquals(0, Character.getType(722513))
    assertEquals(0, Character.getType(725977))
    assertEquals(0, Character.getType(726721))
    assertEquals(0, Character.getType(727866))
    assertEquals(0, Character.getType(729810))
    assertEquals(0, Character.getType(730838))
    assertEquals(0, Character.getType(730973))
    assertEquals(0, Character.getType(733586))
    assertEquals(0, Character.getType(740086))
    assertEquals(0, Character.getType(743213))
    assertEquals(0, Character.getType(744503))
    assertEquals(0, Character.getType(745502))
    assertEquals(0, Character.getType(748676))
    assertEquals(0, Character.getType(757589))
    assertEquals(0, Character.getType(760894))
    assertEquals(0, Character.getType(761113))
    assertEquals(0, Character.getType(762701))
    assertEquals(0, Character.getType(764666))
    assertEquals(0, Character.getType(769454))
    assertEquals(0, Character.getType(779853))
    assertEquals(0, Character.getType(780781))
    assertEquals(0, Character.getType(781852))
    assertEquals(0, Character.getType(781978))
    assertEquals(0, Character.getType(785688))
    assertEquals(0, Character.getType(786900))
    assertEquals(0, Character.getType(790949))
    assertEquals(0, Character.getType(791871))
    assertEquals(0, Character.getType(793467))
    assertEquals(0, Character.getType(801002))
    assertEquals(0, Character.getType(804941))
    assertEquals(0, Character.getType(805357))
    assertEquals(0, Character.getType(818312))
    assertEquals(0, Character.getType(819974))
    assertEquals(0, Character.getType(819991))
    assertEquals(0, Character.getType(820603))
    assertEquals(0, Character.getType(825262))
    assertEquals(0, Character.getType(825983))
    assertEquals(0, Character.getType(826693))
    assertEquals(0, Character.getType(835921))
    assertEquals(0, Character.getType(836149))
    assertEquals(0, Character.getType(836435))
    assertEquals(0, Character.getType(837216))
    assertEquals(0, Character.getType(839648))
    assertEquals(0, Character.getType(842471))
    assertEquals(0, Character.getType(845710))
    assertEquals(0, Character.getType(847881))
    assertEquals(0, Character.getType(848209))
    assertEquals(0, Character.getType(848861))
    assertEquals(0, Character.getType(856409))
    assertEquals(0, Character.getType(863204))
    assertEquals(0, Character.getType(865730))
    assertEquals(0, Character.getType(867884))
    assertEquals(0, Character.getType(869447))
    assertEquals(0, Character.getType(872726))
    assertEquals(0, Character.getType(877063))
    assertEquals(0, Character.getType(877663))
    assertEquals(0, Character.getType(879020))
    assertEquals(0, Character.getType(880659))
    assertEquals(0, Character.getType(883941))
    assertEquals(0, Character.getType(888825))
    assertEquals(0, Character.getType(891451))
    assertEquals(0, Character.getType(899396))
    assertEquals(0, Character.getType(902996))
    assertEquals(0, Character.getType(904030))
    assertEquals(0, Character.getType(904638))
    assertEquals(0, Character.getType(910762))
    assertEquals(0, Character.getType(913805))
    assertEquals(0, Character.getType(916889))
    assertEquals(0, Character.getType(918899))
    assertEquals(0, Character.getType(920560))
    assertEquals(0, Character.getType(923643))
    assertEquals(0, Character.getType(927674))
    assertEquals(0, Character.getType(941501))
    assertEquals(0, Character.getType(942293))
    assertEquals(0, Character.getType(942398))
    assertEquals(0, Character.getType(947312))
    assertEquals(0, Character.getType(947843))
    assertEquals(0, Character.getType(954629))
    assertEquals(0, Character.getType(957954))
    assertEquals(0, Character.getType(957997))
    assertEquals(0, Character.getType(958413))
    assertEquals(0, Character.getType(959866))
    assertEquals(0, Character.getType(960082))
    assertEquals(0, Character.getType(960943))
    assertEquals(0, Character.getType(966692))
    assertEquals(0, Character.getType(973576))
    assertEquals(0, Character.getType(975710))
    assertEquals(0, Character.getType(976432))
    assertEquals(0, Character.getType(979277))
    assertEquals(0, Character.getType(981840))
    assertEquals(0, Character.getType(981940))
    assertEquals(0, Character.getType(982487))
    assertEquals(18, Character.getType(985623))
    assertEquals(18, Character.getType(989149))
    assertEquals(18, Character.getType(997988))
    assertEquals(18, Character.getType(1000132))
    assertEquals(18, Character.getType(1002215))
    assertEquals(18, Character.getType(1002917))
    assertEquals(18, Character.getType(1004580))
    assertEquals(18, Character.getType(1006595))
    assertEquals(18, Character.getType(1007725))
    assertEquals(18, Character.getType(1010481))
    assertEquals(18, Character.getType(1016838))
    assertEquals(18, Character.getType(1016895))
    assertEquals(18, Character.getType(1018160))
    assertEquals(18, Character.getType(1020126))
    assertEquals(18, Character.getType(1026576))
    assertEquals(18, Character.getType(1027582))
    assertEquals(18, Character.getType(1036147))
    assertEquals(18, Character.getType(1036943))
    assertEquals(18, Character.getType(1040859))
    assertEquals(18, Character.getType(1041209))
    assertEquals(18, Character.getType(1044637))
    assertEquals(18, Character.getType(1045872))
    assertEquals(18, Character.getType(1045977))
    assertEquals(18, Character.getType(1052466))
    assertEquals(18, Character.getType(1052541))
    assertEquals(18, Character.getType(1052557))
    assertEquals(18, Character.getType(1052647))
    assertEquals(18, Character.getType(1060162))
    assertEquals(18, Character.getType(1065690))
    assertEquals(18, Character.getType(1078626))
    assertEquals(18, Character.getType(1080764))
    assertEquals(18, Character.getType(1080771))
    assertEquals(18, Character.getType(1082986))
    assertEquals(18, Character.getType(1083091))
    assertEquals(18, Character.getType(1086585))
    assertEquals(18, Character.getType(1087471))
    assertEquals(18, Character.getType(1094259))
    assertEquals(18, Character.getType(1097626))
    assertEquals(18, Character.getType(1101125))
    assertEquals(18, Character.getType(1103470))
    assertEquals(18, Character.getType(1106648))
    assertEquals(18, Character.getType(1107521))
    assertEquals(18, Character.getType(1112476))
  }

  @Test def isJavaLetter(): Unit = {
    // 50 randomly chosen characters that produce true
    assertTrue(Character.isJavaLetter('\u1115'))
    assertTrue(Character.isJavaLetter('\u1895'))
    assertTrue(Character.isJavaLetter('\u1E89'))
    assertTrue(Character.isJavaLetter('\u1ECF'))
    assertTrue(Character.isJavaLetter('\u3C72'))
    assertTrue(Character.isJavaLetter('\u3D16'))
    assertTrue(Character.isJavaLetter('\u426E'))
    assertTrue(Character.isJavaLetter('\u436D'))
    assertTrue(Character.isJavaLetter('\u4723'))
    assertTrue(Character.isJavaLetter('\u500F'))
    assertTrue(Character.isJavaLetter('\u52D3'))
    assertTrue(Character.isJavaLetter('\u55DA'))
    assertTrue(Character.isJavaLetter('\u571D'))
    assertTrue(Character.isJavaLetter('\u5CE6'))
    assertTrue(Character.isJavaLetter('\u5F2A'))
    assertTrue(Character.isJavaLetter('\u5F3B'))
    assertTrue(Character.isJavaLetter('\u60ED'))
    assertTrue(Character.isJavaLetter('\u611A'))
    assertTrue(Character.isJavaLetter('\u626A'))
    assertTrue(Character.isJavaLetter('\u6539'))
    assertTrue(Character.isJavaLetter('\u6609'))
    assertTrue(Character.isJavaLetter('\u0698'))
    assertTrue(Character.isJavaLetter('\u6FF6'))
    assertTrue(Character.isJavaLetter('\u7332'))
    assertTrue(Character.isJavaLetter('\u7347'))
    assertTrue(Character.isJavaLetter('\u7431'))
    assertTrue(Character.isJavaLetter('\u7F68'))
    assertTrue(Character.isJavaLetter('\u7FB9'))
    assertTrue(Character.isJavaLetter('\u7FC0'))
    assertTrue(Character.isJavaLetter('\u8603'))
    assertTrue(Character.isJavaLetter('\u8707'))
    assertTrue(Character.isJavaLetter('\u8871'))
    assertTrue(Character.isJavaLetter('\u8B1B'))
    assertTrue(Character.isJavaLetter('\u8E1B'))
    assertTrue(Character.isJavaLetter('\u9176'))
    assertTrue(Character.isJavaLetter('\u99D5'))
    assertTrue(Character.isJavaLetter('\u9D5E'))
    assertTrue(Character.isJavaLetter('\u9D62'))
    assertTrue(Character.isJavaLetter('\uA0E2'))
    if (!executingInJVMOnJDK6) {
      assertTrue(Character.isJavaLetter('\uA61E'))
      assertTrue(Character.isJavaLetter('\uA6C3'))
    }
    assertTrue(Character.isJavaLetter('\uAFAD'))
    assertTrue(Character.isJavaLetter('\uBB42'))
    assertTrue(Character.isJavaLetter('\uC4B5'))
    assertTrue(Character.isJavaLetter('\uC4DA'))
    assertTrue(Character.isJavaLetter('\uD301'))
    assertTrue(Character.isJavaLetter('\uD3E5'))
    assertTrue(Character.isJavaLetter('\u0EAB'))
    assertTrue(Character.isJavaLetter('\uF9C0'))
    assertTrue(Character.isJavaLetter('\uFD8F'))

    // 50 randomly chosen characters that produce false
    assertFalse(Character.isJavaLetter('\u19CF'))
    assertFalse(Character.isJavaLetter('\u23D7'))
    assertFalse(Character.isJavaLetter('\u26A9'))
    assertFalse(Character.isJavaLetter('\u2775'))
    assertFalse(Character.isJavaLetter('\u2832'))
    assertFalse(Character.isJavaLetter('\u2885'))
    assertFalse(Character.isJavaLetter('\u2930'))
    assertFalse(Character.isJavaLetter('\u2B8F'))
    assertFalse(Character.isJavaLetter('\u2CF4'))
    assertFalse(Character.isJavaLetter('\u2F70'))
    assertFalse(Character.isJavaLetter('\u0037'))
    assertFalse(Character.isJavaLetter('\u073F'))
    assertFalse(Character.isJavaLetter('\u0969'))
    assertFalse(Character.isJavaLetter('\u9FF3'))
    assertFalse(Character.isJavaLetter('\uA9E5'))
    assertFalse(Character.isJavaLetter('\uAAC1'))
    assertFalse(Character.isJavaLetter('\uAAF7'))
    assertFalse(Character.isJavaLetter('\u0B82'))
    assertFalse(Character.isJavaLetter('\uDA08'))
    assertFalse(Character.isJavaLetter('\uDA8D'))
    assertFalse(Character.isJavaLetter('\uDBC8'))
    assertFalse(Character.isJavaLetter('\uDBF7'))
    assertFalse(Character.isJavaLetter('\uDD3C'))
    assertFalse(Character.isJavaLetter('\uDDD1'))
    assertFalse(Character.isJavaLetter('\uDEC2'))
    assertFalse(Character.isJavaLetter('\uE068'))
    assertFalse(Character.isJavaLetter('\uE0A3'))
    assertFalse(Character.isJavaLetter('\uE210'))
    assertFalse(Character.isJavaLetter('\uE414'))
    assertFalse(Character.isJavaLetter('\u0E4C'))
    assertFalse(Character.isJavaLetter('\uE4FD'))
    assertFalse(Character.isJavaLetter('\uE5F7'))
    assertFalse(Character.isJavaLetter('\uE611'))
    assertFalse(Character.isJavaLetter('\u0E6A'))
    assertFalse(Character.isJavaLetter('\uE704'))
    assertFalse(Character.isJavaLetter('\uE71F'))
    assertFalse(Character.isJavaLetter('\uE7DB'))
    assertFalse(Character.isJavaLetter('\uEA9F'))
    assertFalse(Character.isJavaLetter('\uEAAB'))
    assertFalse(Character.isJavaLetter('\uEBF1'))
    assertFalse(Character.isJavaLetter('\uED24'))
    assertFalse(Character.isJavaLetter('\uED93'))
    assertFalse(Character.isJavaLetter('\uEDE1'))
    assertFalse(Character.isJavaLetter('\uEF40'))
    assertFalse(Character.isJavaLetter('\uF312'))
    assertFalse(Character.isJavaLetter('\uF6B9'))
    assertFalse(Character.isJavaLetter('\uF80F'))
    assertFalse(Character.isJavaLetter('\uFD3E'))
    assertFalse(Character.isJavaLetter('\u0FF9'))
  }

  @Test def isJavaLetterOrDigit(): Unit = {
    // 50 randomly chosen characters that produce true
    assertTrue(Character.isJavaLetterOrDigit('\u0B42'))
    assertTrue(Character.isJavaLetterOrDigit('\u0C35'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isJavaLetterOrDigit('\u1DBD'))
    assertTrue(Character.isJavaLetterOrDigit('\u1FE5'))
    assertTrue(Character.isJavaLetterOrDigit('\u3572'))
    assertTrue(Character.isJavaLetterOrDigit('\u3FBC'))
    assertTrue(Character.isJavaLetterOrDigit('\u46E6'))
    assertTrue(Character.isJavaLetterOrDigit('\u47C1'))
    assertTrue(Character.isJavaLetterOrDigit('\u495B'))
    assertTrue(Character.isJavaLetterOrDigit('\u49D3'))
    assertTrue(Character.isJavaLetterOrDigit('\u4E38'))
    assertTrue(Character.isJavaLetterOrDigit('\u5513'))
    assertTrue(Character.isJavaLetterOrDigit('\u57CD'))
    assertTrue(Character.isJavaLetterOrDigit('\u5884'))
    assertTrue(Character.isJavaLetterOrDigit('\u5A74'))
    assertTrue(Character.isJavaLetterOrDigit('\u5CEB'))
    assertTrue(Character.isJavaLetterOrDigit('\u5D4B'))
    assertTrue(Character.isJavaLetterOrDigit('\u5F0B'))
    assertTrue(Character.isJavaLetterOrDigit('\u6889'))
    assertTrue(Character.isJavaLetterOrDigit('\u69D0'))
    assertTrue(Character.isJavaLetterOrDigit('\u6A40'))
    assertTrue(Character.isJavaLetterOrDigit('\u6C9E'))
    assertTrue(Character.isJavaLetterOrDigit('\u7237'))
    assertTrue(Character.isJavaLetterOrDigit('\u7289'))
    assertTrue(Character.isJavaLetterOrDigit('\u7313'))
    assertTrue(Character.isJavaLetterOrDigit('\u7626'))
    assertTrue(Character.isJavaLetterOrDigit('\u7C1C'))
    assertTrue(Character.isJavaLetterOrDigit('\u7DC9'))
    assertTrue(Character.isJavaLetterOrDigit('\u8093'))
    assertTrue(Character.isJavaLetterOrDigit('\u8573'))
    assertTrue(Character.isJavaLetterOrDigit('\u884E'))
    assertTrue(Character.isJavaLetterOrDigit('\u90AE'))
    assertTrue(Character.isJavaLetterOrDigit('\u94E8'))
    assertTrue(Character.isJavaLetterOrDigit('\u956A'))
    assertTrue(Character.isJavaLetterOrDigit('\u9912'))
    assertTrue(Character.isJavaLetterOrDigit('\u9E2A'))
    assertTrue(Character.isJavaLetterOrDigit('\uACE6'))
    assertTrue(Character.isJavaLetterOrDigit('\uB0E7'))
    assertTrue(Character.isJavaLetterOrDigit('\uB3E5'))
    assertTrue(Character.isJavaLetterOrDigit('\uB7CA'))
    assertTrue(Character.isJavaLetterOrDigit('\uBD2C'))
    assertTrue(Character.isJavaLetterOrDigit('\uBD3F'))
    assertTrue(Character.isJavaLetterOrDigit('\uC155'))
    assertTrue(Character.isJavaLetterOrDigit('\uC585'))
    assertTrue(Character.isJavaLetterOrDigit('\uC7FC'))
    assertTrue(Character.isJavaLetterOrDigit('\uD250'))
    assertTrue(Character.isJavaLetterOrDigit('\uD378'))
    assertTrue(Character.isJavaLetterOrDigit('\uD73B'))
    assertTrue(Character.isJavaLetterOrDigit('\uFDB1'))
    assertTrue(Character.isJavaLetterOrDigit('\uFEE5'))

    // 50 randomly chosen characters that produce false
    assertFalse(Character.isJavaLetterOrDigit('\u05FD'))
    assertFalse(Character.isJavaLetterOrDigit('\u08B3'))
    assertFalse(Character.isJavaLetterOrDigit('\u09FE'))
    assertFalse(Character.isJavaLetterOrDigit('\u0AD6'))
    assertFalse(Character.isJavaLetterOrDigit('\u0F3D'))
    assertFalse(Character.isJavaLetterOrDigit('\u0FCA'))
    assertFalse(Character.isJavaLetterOrDigit('\u17F5'))
    assertFalse(Character.isJavaLetterOrDigit('\u1A5F'))
    assertFalse(Character.isJavaLetterOrDigit('\u1F4E'))
    assertFalse(Character.isJavaLetterOrDigit('\u204C'))
    assertFalse(Character.isJavaLetterOrDigit('\u224A'))
    assertFalse(Character.isJavaLetterOrDigit('\u2296'))
    assertFalse(Character.isJavaLetterOrDigit('\u241C'))
    assertFalse(Character.isJavaLetterOrDigit('\u247C'))
    assertFalse(Character.isJavaLetterOrDigit('\u2490'))
    assertFalse(Character.isJavaLetterOrDigit('\u24B8'))
    assertFalse(Character.isJavaLetterOrDigit('\u253B'))
    assertFalse(Character.isJavaLetterOrDigit('\u2755'))
    assertFalse(Character.isJavaLetterOrDigit('\u2A18'))
    assertFalse(Character.isJavaLetterOrDigit('\u2A64'))
    assertFalse(Character.isJavaLetterOrDigit('\u2A91'))
    assertFalse(Character.isJavaLetterOrDigit('\u2F0F'))
    assertFalse(Character.isJavaLetterOrDigit('\uA4C1'))
    assertFalse(Character.isJavaLetterOrDigit('\uA4C3'))
    assertFalse(Character.isJavaLetterOrDigit('\uA797'))
    assertFalse(Character.isJavaLetterOrDigit('\uA7C3'))
    assertFalse(Character.isJavaLetterOrDigit('\uDA09'))
    assertFalse(Character.isJavaLetterOrDigit('\uDB3E'))
    assertFalse(Character.isJavaLetterOrDigit('\uDE30'))
    assertFalse(Character.isJavaLetterOrDigit('\uDFB4'))
    assertFalse(Character.isJavaLetterOrDigit('\uE116'))
    assertFalse(Character.isJavaLetterOrDigit('\uE20D'))
    assertFalse(Character.isJavaLetterOrDigit('\uE21D'))
    assertFalse(Character.isJavaLetterOrDigit('\uE341'))
    assertFalse(Character.isJavaLetterOrDigit('\uE4DC'))
    assertFalse(Character.isJavaLetterOrDigit('\uE6F7'))
    assertFalse(Character.isJavaLetterOrDigit('\uE6FB'))
    assertFalse(Character.isJavaLetterOrDigit('\uE785'))
    assertFalse(Character.isJavaLetterOrDigit('\uEA37'))
    assertFalse(Character.isJavaLetterOrDigit('\uEB1C'))
    assertFalse(Character.isJavaLetterOrDigit('\uEE47'))
    assertFalse(Character.isJavaLetterOrDigit('\uEEA6'))
    assertFalse(Character.isJavaLetterOrDigit('\uF27F'))
    assertFalse(Character.isJavaLetterOrDigit('\uF2FE'))
    assertFalse(Character.isJavaLetterOrDigit('\uF33F'))
    assertFalse(Character.isJavaLetterOrDigit('\uF3B1'))
    assertFalse(Character.isJavaLetterOrDigit('\uF5AD'))
    assertFalse(Character.isJavaLetterOrDigit('\uF6B1'))
    assertFalse(Character.isJavaLetterOrDigit('\uF84B'))
    assertFalse(Character.isJavaLetterOrDigit('\uFF03'))
  }

  @Test def isJavaIdentifierStart(): Unit = {
    // 50 randomly chosen characters that produce true
    assertTrue(Character.isJavaIdentifierStart('\u186C'))
    assertTrue(Character.isJavaIdentifierStart('\u1F75'))
    assertTrue(Character.isJavaIdentifierStart('\u347C'))
    assertTrue(Character.isJavaIdentifierStart('\u485F'))
    assertTrue(Character.isJavaIdentifierStart('\u54A6'))
    assertTrue(Character.isJavaIdentifierStart('\u58CF'))
    assertTrue(Character.isJavaIdentifierStart('\u65F6'))
    assertTrue(Character.isJavaIdentifierStart('\u6E34'))
    assertTrue(Character.isJavaIdentifierStart('\u7404'))
    assertTrue(Character.isJavaIdentifierStart('\u77BD'))
    assertTrue(Character.isJavaIdentifierStart('\u7977'))
    assertTrue(Character.isJavaIdentifierStart('\u83A9'))
    assertTrue(Character.isJavaIdentifierStart('\u844E'))
    assertTrue(Character.isJavaIdentifierStart('\u9D7D'))
    assertTrue(Character.isJavaIdentifierStart('\u9E7F'))
    if (!executingInJVMOnJDK6) {
      assertTrue(Character.isJavaIdentifierStart('\uA734'))
      assertTrue(Character.isJavaIdentifierStart('\uAA81'))
    }
    assertTrue(Character.isJavaIdentifierStart('\uACED'))
    assertTrue(Character.isJavaIdentifierStart('\uBD6B'))
    assertTrue(Character.isJavaIdentifierStart('\uBD89'))
    assertTrue(Character.isJavaIdentifierStart('\uC5E4'))
    assertTrue(Character.isJavaIdentifierStart('\uC7C7'))
    assertTrue(Character.isJavaIdentifierStart('\uD3C8'))
    assertTrue(Character.isJavaIdentifierStart('\uF972'))
    if (!executingInJVMOnJDK6)
      assertTrue(Character.isJavaIdentifierStart('\uFA9F'))
    assertTrue(Character.isJavaIdentifierStart('\uFDA6'))
    assertTrue(Character.isJavaIdentifierStart(132773))
    assertTrue(Character.isJavaIdentifierStart(136707))
    assertTrue(Character.isJavaIdentifierStart(140254))
    assertTrue(Character.isJavaIdentifierStart(141336))
    assertTrue(Character.isJavaIdentifierStart(147781))
    assertTrue(Character.isJavaIdentifierStart(148525))
    assertTrue(Character.isJavaIdentifierStart(149236))
    assertTrue(Character.isJavaIdentifierStart(151919))
    assertTrue(Character.isJavaIdentifierStart(153364))
    assertTrue(Character.isJavaIdentifierStart(153898))
    assertTrue(Character.isJavaIdentifierStart(157645))
    assertTrue(Character.isJavaIdentifierStart(158882))
    assertTrue(Character.isJavaIdentifierStart(161627))
    assertTrue(Character.isJavaIdentifierStart(164324))
    assertTrue(Character.isJavaIdentifierStart(168371))
    assertTrue(Character.isJavaIdentifierStart(169573))
    assertTrue(Character.isJavaIdentifierStart(171871))
    assertTrue(Character.isJavaIdentifierStart(172017))
    assertTrue(Character.isJavaIdentifierStart(172787))
    assertTrue(Character.isJavaIdentifierStart(173670))
    if (!executingInJVMOnJDK6) {
      assertTrue(Character.isJavaIdentifierStart(175824))
      assertTrue(Character.isJavaIdentifierStart(74462))
      assertTrue(Character.isJavaIdentifierStart(78380))
    }

    // 50 randomly chosen characters that produce false
    assertFalse(Character.isJavaIdentifierStart('\uE8A1'))
    assertFalse(Character.isJavaIdentifierStart('\uEBB6'))
    assertFalse(Character.isJavaIdentifierStart(1006625))
    assertFalse(Character.isJavaIdentifierStart(1038096))
    assertFalse(Character.isJavaIdentifierStart(1080963))
    assertFalse(Character.isJavaIdentifierStart(1098681))
    assertFalse(Character.isJavaIdentifierStart(1101181))
    assertFalse(Character.isJavaIdentifierStart(185775))
    assertFalse(Character.isJavaIdentifierStart(200078))
    assertFalse(Character.isJavaIdentifierStart(212082))
    assertFalse(Character.isJavaIdentifierStart(219401))
    assertFalse(Character.isJavaIdentifierStart(223687))
    assertFalse(Character.isJavaIdentifierStart(230929))
    assertFalse(Character.isJavaIdentifierStart(238776))
    assertFalse(Character.isJavaIdentifierStart(275525))
    assertFalse(Character.isJavaIdentifierStart(297540))
    assertFalse(Character.isJavaIdentifierStart(310684))
    assertFalse(Character.isJavaIdentifierStart(323571))
    assertFalse(Character.isJavaIdentifierStart(340895))
    assertFalse(Character.isJavaIdentifierStart(355972))
    assertFalse(Character.isJavaIdentifierStart(382586))
    assertFalse(Character.isJavaIdentifierStart(385597))
    assertFalse(Character.isJavaIdentifierStart(438811))
    assertFalse(Character.isJavaIdentifierStart(442235))
    assertFalse(Character.isJavaIdentifierStart(446274))
    assertFalse(Character.isJavaIdentifierStart(454075))
    assertFalse(Character.isJavaIdentifierStart(464326))
    assertFalse(Character.isJavaIdentifierStart(469494))
    assertFalse(Character.isJavaIdentifierStart(473583))
    assertFalse(Character.isJavaIdentifierStart(481617))
    assertFalse(Character.isJavaIdentifierStart(492310))
    assertFalse(Character.isJavaIdentifierStart(502865))
    assertFalse(Character.isJavaIdentifierStart(505834))
    assertFalse(Character.isJavaIdentifierStart(521762))
    assertFalse(Character.isJavaIdentifierStart(531102))
    assertFalse(Character.isJavaIdentifierStart(531557))
    assertFalse(Character.isJavaIdentifierStart(573907))
    assertFalse(Character.isJavaIdentifierStart(590651))
    assertFalse(Character.isJavaIdentifierStart(602729))
    assertFalse(Character.isJavaIdentifierStart(607107))
    assertFalse(Character.isJavaIdentifierStart(657499))
    assertFalse(Character.isJavaIdentifierStart(684199))
    assertFalse(Character.isJavaIdentifierStart(727324))
    assertFalse(Character.isJavaIdentifierStart(757790))
    assertFalse(Character.isJavaIdentifierStart(783173))
    assertFalse(Character.isJavaIdentifierStart(805578))
    assertFalse(Character.isJavaIdentifierStart(834136))
    assertFalse(Character.isJavaIdentifierStart(875116))
    assertFalse(Character.isJavaIdentifierStart(955913))
    assertFalse(Character.isJavaIdentifierStart(958246))
  }
}
